#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <chrono>

#include "clases/Mundo.h"
#include "clases/Entidad.h"
#include "clases/Vehiculo.h"
#include "clases/MapObject.h"
#include "clases/Nivel.h"
#include "clases/3DLoader.h"
#include "clases/HUD.h"
#include "clases/ContactListener.h"

#include "SDL.h"
#include "SDL_opengl.h"
#include "SDL_ttf.h"
#include <gl/GLU.h>
#include "Box2D/Box2D.h"
#include "pugixml-1.7/src/pugixml.hpp"

#define MS_POR_FRAME 8     //125 fps
#define MAX_UPDATES_POR_FRAME 5		//un numero arbitrario


//#define NO_RENDER_MODELO false //esto lo uso para que no cargue el modelo asi se abre mas rapido

using namespace std;

// CONSTANTES DE CONFIGURACION
// Pantalla y FPS
int ANCHO = 1300;
int LARGO = 700;
int BPP = 32;
float SPF = 1/30.f;


// DIRECTORIOS
string MODELOS = "modelos/";
string NIVELES = "niveles/";


void actualizarCam(float &x, float &y, float &z, float x_angle, float y_angle, float radius) {
	z = cos(y_angle*M_PI / 180) * cos(x_angle*M_PI / 180) * radius;
	x = sin(y_angle*M_PI / 180) * cos(x_angle*M_PI / 180) * radius;
	y = sin(x_angle*M_PI / 180) * radius;
}

void lightSwitch(bool lightEnabled, GLenum light_id) {
	if (lightEnabled)
		glEnable(light_id);
	else
		glDisable(light_id);
}

void lightSet(GLenum light_id, GLfloat* colorAmbiente, GLfloat* colorDifuso, GLfloat* colorSpecular) {
	glLightfv(light_id, GL_DIFFUSE, colorDifuso);
	glLightfv(light_id, GL_AMBIENT, colorAmbiente);
	glLightfv(light_id, GL_SPECULAR, colorSpecular);
}

int main(int argc, char *argv[]) {
    if(SDL_Init(SDL_INIT_VIDEO)<0) {
        cerr << "No se pudo iniciar SDL: " << SDL_GetError() << endl;
        exit(1);
    }
    atexit(SDL_Quit);
    Uint32 flags = /*SDL_DOUBLEBUF | SDL_HWSURFACE |*/ SDL_WINDOW_OPENGL;

	SDL_Window *window;                    // Declare a pointer

	SDL_Init(SDL_INIT_VIDEO);              // Initialize SDL2

	window = SDL_CreateWindow(
		"Free Rider 3D",                  // window title
		SDL_WINDOWPOS_UNDEFINED,           // initial x position
		SDL_WINDOWPOS_UNDEFINED,           // initial y position
		ANCHO,                               // width, in pixels
		LARGO,                               // height, in pixels
		SDL_WINDOW_OPENGL                  // flags - see below
		);


    if(window == NULL) {
        cerr << "No se pudo establecer el modo de video: " << SDL_GetError() << endl;
        exit(1);
    }

	SDL_GLContext glContext = SDL_GL_CreateContext(window);
	if (window == NULL) {
		fprintf(stderr, "[GL Context Error]: %s\n", SDL_GetError());
		SDL_Quit();
		exit(1);
	}


	//Inicializacion de SDL_ttf
	if (TTF_Init() < 0) {
		cerr << "No se pudo iniciar SDL_ttf: " << SDL_GetError() << endl;
		exit(1);
		int i = 0;
	}

	//Se carga la fuente del texto y el color
	TTF_Font *fuente = TTF_OpenFont("BEBAS___.ttf", 8);
	TTF_Font *fuenteAyuda = TTF_OpenFont("BEBAS___.ttf", 10);
	
	SDL_Color colorAyuda = { 0,255,0 };
	SDL_Color colorfuente = { 1,1,255 };

	//Textura que va a utilizar el texto
	GLuint TextureID = 0;
	glGenTextures(1, &TextureID);
	glBindTexture(GL_TEXTURE_2D, TextureID);

	//Nombre del nivel
	string nombreNivel;

	//Inicializacion del puntaje
	int puntaje = 0;

	//inicializacion de booleanos de ayuda
	bool ayuda = false;
	bool wireframe = false;
	bool texturas = true;
	float velJuego = 1.0;

	//Wireframe
	GLenum tipo = GL_FILL;

    float color = 1.0;
    glClearColor(0.529f, 0.807f, 0.980f, 1);

    SDL_Event evento;
    bool fin = false;

	//cosas del box2D
	float32 timeStep = MS_POR_FRAME / 1000.0f;
	int32 velocityIterations = 8;
	int32 positionIterations = 3;
	
	Vehiculo * moto;

	// Crear Moto
	moto = new Vehiculo(*parsearVehiculo(MODELOS + "warbike/", "bike.obj",2.0f));

	ContactListener contactListener(moto);
	mundo.SetContactListener(&contactListener);

    // Cargar Nivel
    Nivel* nivel = new Nivel (NIVELES + "nivel01.xml", &contactListener, moto);



    // Control Temporal
	chrono::steady_clock::time_point tiempoAnterior = chrono::steady_clock::now();
	chrono::steady_clock::time_point tiempoActual;
	chrono::steady_clock::duration tiempoTranscurrido;
	chrono::steady_clock::duration lag = chrono::milliseconds(0);
	chrono::steady_clock::duration ms_por_update = chrono::milliseconds(MS_POR_FRAME);


    //pausa
	bool pausa = false;

	//control de camara
	bool moverCam = false;
	float anga = 0;
	float angb = 0;
	float angulo = 0;
	float rad = -13;
	float x = 0;
	float y = 0.0;
	float z = rad;

	//luces                              0                           1                  2                          3                    4                      5                   6                     7
	GLfloat lightPos[8][4] = { { 0.0, 15.0, 0.0, 1.0 },{ 30.0, 5.0, 0.0, 1.0 },{ 0.0, 0.0, 0.0, 1.0 },{ 0.0, 0.0, 0.0, 1.0 },{ 0.0, 5.0, 1.0, 0.0 },{ 600, 30.0, 0.0, 1.0 },{ 0.0, 0.0, 0.0, 1.0 },{ 0.0, 0.0, 0.0, 1.0 } };
	GLfloat lightColorDif[8][4] = { { 1.0, 1.0, 1.0, 1.0 },{ 1.0, 0.0, 0.0, 1.0 },{ 0.0, 1.0, 0.0, 1.0 },{ 1.0, 0.0, 0.0, 1.0 },{ 1.0, 1.0, 1.0, 1.0 },{ 1.0, 1.0, 1.0, 1.0 },{ 0.0, 0.5, 0.9, 1.0 },{ 1.0, 0.2, 0.1, 1.0 } };
	GLfloat lightColorAmb[8][4] = { { 0.3, 0.3, 0.3, 1.0 },{ 0.3, 0.1, 0, 1.0 },{ 0.6, 0.1, 0.9, 1.0 },{ 0.0, 0.0, 0.2, 1.0 },{ 0.3, 0.3, 0.3, 1.0 },{ 0.3, 0.3, 0.3, 1.0 },{ 0.0, 0.6, 0.2, 1.0 },{ 0.3, 0.1, 0.6, 1.0 } };
	GLfloat lightColorSpe[8][4] = { { 0.0, 0.0, 0.0, 1.0 },{ 0.3, 0.1, 0, 1.0 },{ 0.1, 0.1, 0.1, 1.0 },{ 0.0, 0.0, 0.0, 1.0 },{ 0.0, 0.0, 0.0, 1.0 },{ 0.0, 0.0, 0.0, 1.0 },{ 0.0, 0.0, 0.0, 1.0 },{ 0.0, 0.0, 0.0, 1.0 }, };
	GLfloat lightSpotDir[8][3] = { { 0.0, -1.0, 0.0 }, {0.0,0.0,-1.0},{ 0.0,0.0,-1.0 },{ 0.0,0.0,-1.0 },{ 0.0,-5.0,-1.0 },{ 0.0,0.0,-1.0 },{ 0.0,0.0,-1.0 },{ 0.0,0.0,-1.0 } };
	bool lightEnabled[8] = { false,false,false,false,false,false,false,false };
	//glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_LIGHTING);

	// Seteo de luces
	for (int i = 0; i < 8; i++) {
		lightSet(GL_LIGHT0 + i, lightColorAmb[i], lightColorDif[i], lightColorSpe[i]);
	}
	lightEnabled[0] = true;
	glEnable(GL_LIGHT0);
	GLint cutoff = 16;
	glLighti(GL_LIGHT0,GL_SPOT_CUTOFF,cutoff);
	glLighti(GL_LIGHT0, GL_SPOT_EXPONENT, 5);

	//lightEnabled[4] = true;
	//glEnable(GL_LIGHT4);
	//lightEnabled[5] = true;
	//glEnable(GL_LIGHT5);
	
	texturasHabilitadas = true;

	int nivelActual = 1;
    while (!fin) {

        tiempoActual = chrono::steady_clock::now();
		tiempoTranscurrido = tiempoActual - tiempoAnterior;
		tiempoAnterior = tiempoActual;
		lag += tiempoTranscurrido;
		//proceso el input
		//TODO meter esto en un procedimiento auxiliar porque es un despelote
		while (SDL_PollEvent(&evento)) {
			switch (evento.type) {
			case SDL_QUIT:
				fin = true;
				break;

			case SDL_KEYDOWN:
				//solo le doy pelota a las teclas sin repeticion
				if (evento.key.repeat == 0) {
					switch (evento.key.keysym.sym) {
					case SDLK_ESCAPE:
						if (ayuda) {
							ayuda = false;
						}
						else {
							ayuda = true;
						}
						break;
					case SDLK_q:
						fin = true;
						break;
					case SDLK_p:
						pausa = !pausa;
						lag = chrono::milliseconds(0);
						break;
					case SDLK_r:
						nivelActual = (nivelActual) % 4 + 1;
						delete nivel;
						switch (nivelActual) {
						case 1:
							nivel = new Nivel(NIVELES + "nivel01.xml", &contactListener, moto);
							break;
						case 2:
							nivel = new Nivel(NIVELES + "nivel02.xml", &contactListener, moto);
							break;
						case 3:
							nivel = new Nivel(NIVELES + "nivel03.xml", &contactListener, moto);
							break;
						case 4:
							nivel = new Nivel(NIVELES + "nivel04.xml", &contactListener, moto);
							break;
						}
						moto->resetNextAct();
						break;						
					case SDLK_w:
						if (wireframe == false) {
							tipo = GL_LINE;
							wireframe = true;
						}
						else {
							tipo = GL_FILL;
							wireframe = false;
						}
						break;
					case SDLK_0:
					case SDLK_1:
					case SDLK_2:
					case SDLK_3:
					case SDLK_4:
					case SDLK_5:
					case SDLK_6:
					case SDLK_7:
					{
						int numluz = evento.key.keysym.sym - SDLK_0;	//0 para SDLK_0, de ahi para arriba
						lightEnabled[numluz] = !lightEnabled[numluz];
						lightSwitch(lightEnabled[numluz], GL_LIGHT0 + numluz);
						break;
					}
					//control de velocidad
					case SDLK_PLUS:
						ms_por_update /= 2;
						velJuego *= 2;
						break;
					case SDLK_MINUS:
						ms_por_update *= 2;
						velJuego /= 2;
						break;
					case SDLK_o:
					case SDLK_l:
						//estas las uso para hacer zoom cuando hay repeat, fijarse mas abajo
						break;
					//estas teclas se se las mando a la moto
					case SDLK_UP:
					case SDLK_DOWN:
					case SDLK_LEFT:
					case SDLK_RIGHT:
						moto->down_key(evento.key.keysym.sym);
						break;
					case SDLK_t:
						texturasHabilitadas = !texturasHabilitadas;
						break;
					}
				}
				else {
					//zoom de la camara
					switch (evento.key.keysym.sym) {
					case SDLK_l:
						rad -= .1;//factor de ajuste: 0,1
						actualizarCam(x, y, z, angb, anga, rad);
						break;
					case SDLK_o:
						if (rad<0)
							rad += .1;//factor de ajuste: 0,1
						actualizarCam(x, y, z, angb, anga, rad);
						break;
					}
				}
				break;

			case SDL_KEYUP:
				switch (evento.key.keysym.sym) {
				//estas teclas se se las mando a la moto
				case SDLK_UP:
				case SDLK_DOWN:
				case SDLK_LEFT:
				case SDLK_RIGHT:
					moto->up_key(evento.key.keysym.sym);
					break;
				case SDLK_SPACE:
					nivel->resetNextAct();
					moto->resetNextAct();
					break;
				}
				break;

			case SDL_MOUSEBUTTONDOWN:
				if (SDL_GetMouseState(NULL, NULL)&SDL_BUTTON(3)) {
					moverCam = true;
				}
				break;

			case SDL_MOUSEBUTTONUP:
				moverCam = false;
				break;

			case SDL_MOUSEMOTION:
				if (moverCam) {
					if (evento.motion.yrel<0 && angb < 80)
						angb -= evento.motion.yrel*0.4;		//factor de ajuste: 0,4
					else if (evento.motion.yrel >= 0 && angb > -80)
						angb -= evento.motion.yrel*0.4;		//factor de ajuste: 0,4
					anga += evento.motion.xrel*0.4;			//factor de ajuste: 0,4
					actualizarCam(x, y, z, angb, anga, rad);
				}
				break;
			}
		}

		//actualizo el mundo
		if (!pausa) {
			int contador_bailout = 0;
			while (lag >= ms_por_update) {
				moto->actualizar();
				nivel->actualizar();
				mundo.Step(timeStep, velocityIterations, positionIterations);
				lag -= ms_por_update;
				if (contador_bailout++ > MAX_UPDATES_POR_FRAME)
					break;	//impido que se quede eternamente aca adentro
			}
		}

		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(45, ANCHO / LARGO, 0.1, 500);
		glEnable(GL_DEPTH_TEST);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();

		//dibujo en pantalla
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glPolygonMode(GL_FRONT_AND_BACK, tipo);

		b2Vec2 posMoto = moto->GetPosition();
		gluLookAt(posMoto.x + x, posMoto.y + y, -z, posMoto.x, posMoto.y + 1, 0, 0, 1, 0);

		//las luces hay que ponerlas despues del mover la camara
		for (int i = 0; i < 8; ++i) {
			glLightfv(GL_LIGHT0 + i, GL_SPOT_DIRECTION, lightSpotDir[i]);
			glLightfv(GL_LIGHT0 + i, GL_POSITION, lightPos[i]);
		}
		if (texturasHabilitadas)
			glEnable(GL_TEXTURE_2D);
		else
			glDisable(GL_TEXTURE_2D);

        moto->dibujar();
        nivel->dibujar();
		
		//Quitar el wireframe para la proyeccion ortogonal
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

		//Obtengo el nombre del nivel
		nombreNivel = nivel->getNombre();
		char *holaMundo = _strdup(nombreNivel.c_str());

		//Dada la posicion le asigno un puntaje
		float pos = posMoto.x;
		if (trunc(pos) > puntaje){
			puntaje = trunc(pos);
		}
		char *punt = _strdup((to_string(puntaje)).c_str());

		b2Vec2 velMoto = moto->GetSpeed();
		float vel = sqrt(pow(velMoto.x, 2) + pow(velMoto.y, 2));
		char *velocidad = _strdup((to_string(vel)).c_str());

		int muertes = moto->getMuertes();
		char *muert = _strdup((to_string(muertes)).c_str());

		string velx = to_string(velJuego) + "X";
		char *juegoVel = _strdup(velx.c_str());

		// Dibujar el HUD con proyeccion ortogonal y determinado tipo de fuente y color
		dibujarHud(fuente, colorfuente, TextureID, holaMundo, punt, velocidad, muert, ayuda, fuenteAyuda, 
					colorAyuda, wireframe, lightEnabled, juegoVel, texturasHabilitadas);

		if (nivel->esVictorioso()) {
			dibujarString("VICTORIA", 50, 255, 1, 1);
		}

        //SDL_GL_SwapBuffers();
		SDL_GL_SwapWindow(window);
    }

	SDL_GL_DeleteContext(glContext);
    return 0;
}
