#pragma once
#ifndef THREEDLOADER_H
#define THREEDLOADER_H

#include <string>
#include <fstream>
#include <vector>
#include <map>
#include <regex>
#include <iostream>

#include "SDL.h"
#include "SDL_opengl.h"
#include "soil\src\SOIL.h"
#include "Mtl.h"
#include "ObjetoGrafico.h"
#include "AngularRotation.h"
#include "Cara.h"

//#include "terna.h"

using namespace std;

map<string,ObjetoGrafico*> cargarMTL(string path, string nombreObj) {
	// Expresiones Regulares
	regex newmtl_rx("^newmtl (.*)");
	regex aColor_rx("\tKa (.+)\\s(.+)\\s(.+)");
	regex dColor_rx("\tKd (.+)\\s(.+)\\s(.+)");
	regex sColorKs_rx("\tKs (.+)\\s(.+)\\s(.+)");
	regex sColorNs_rx("\tNs (.+)");
	regex trans_rx("\t(d|Tr) (.+)");
	regex illum_rx("\tillum (.+)");
	regex texMap_rx("\tmap_Kd (.+)");
	// Variables
	map<string, ObjetoGrafico*> retorno;
	float ambientColor[3];
	float diffuseColor[3];
	float specularColor[4];
	float transparency;
	int illum;
	string mapKd;
	GLuint texID;
	string linea = "";
	string nombreMtl = "";
	// Arreglo de matcheo
	smatch matcheo;
	// Abrir archivo
	ifstream archivo;
	archivo.open(path + nombreObj);
	// Leer archivo
	while (getline(archivo, linea)) {
		// Parsear linea
		if (regex_match(linea,matcheo,newmtl_rx)) {
			if (matcheo.size() >= 2) {
				nombreMtl = matcheo[1];
			}
		}
		else if (regex_match(linea, matcheo, aColor_rx)) {
			if (matcheo.size() >= 4) {
				for (unsigned i = 1; i < 4; i++)
					ambientColor[i - 1] = stof(matcheo[i]);
			}
		}
		else if (regex_match(linea, matcheo, dColor_rx)) {
			if (matcheo.size() >= 4) {
				for (unsigned i = 1; i < 4; i++)
					diffuseColor[i - 1] = stof(matcheo[i]);
			}
		}
		else if (regex_match(linea, matcheo, sColorKs_rx)) {
			if (matcheo.size() >= 4) {
				for (unsigned i = 1; i < 4; i++)
					specularColor[i - 1] = stof(matcheo[i]);
			}
		}
		else if (regex_match(linea, matcheo, sColorNs_rx)) {
			if (matcheo.size() >= 2) {
				specularColor[3] = stof(matcheo[1]);
			}
		}
		else if (regex_match(linea, matcheo, trans_rx)) {
			if (matcheo.size() >= 3) {
				transparency = stof(matcheo[2]);
			}
		}
		else if (regex_match(linea, matcheo, illum_rx)) {
			if (matcheo.size() >= 2) {
				illum = stoi(matcheo[1]);
			}
		}
		else if (regex_match(linea, matcheo, texMap_rx)) {
			if (matcheo.size() >= 2) {
				mapKd = matcheo[1];				
				ObjetoGrafico* ingreso = new Mtl(nombreMtl, ambientColor, diffuseColor, specularColor, transparency, illum, mapKd, path + matcheo[1].str());
				retorno[nombreMtl] = ingreso;
			}
		}
	}
	archivo.close();
	return retorno;
};

/* Parsea un archivo .obj y genera un vector de ternas que describe al objeto
 * Los archivos .obj disponen de 6 comandos posibles, listados a continuacin:
 * ['#' + .*] es un comentario
 * [('usemtl' | 'mtllib') .*] son tiles para cargar texturas
 * ['v' float float float] describe un vrtice
 * ['vt' float float] describe una posicin relativa sobre una textura 2D
 * ['vn' float float float] describe la normal de un vrtice
 * ['f' \d+/\d+/\d+ \d+/\d+/\d+ \d+/\d+/\d+] describe una cara.
 *
 * Las caras se definen mediante tringulos. Cada bloque corresponde a #vertice/#textura/#normal,
 * donde cada nmero se asocia segn el orden en el que fueron definidas las estructuras (comienza en 1).
*/
vector<ObjetoGrafico*>* parsearObj(string path, string nombreObj, bool &repetir, float mult = 1.0){
	// Expresiones Regulares
	regex usemtl_rx ("^usemtl (.*)");
	regex mtllib_rx ("^mtllib (.*)");
	regex pos_rx("^pos\\s(.+)\\s(.+)\\s(.+)");
	regex elemento_rx ("^(v|vt|vn|f)\\s(.+)\\s(.+)\\s(.+)");
	regex tex_rx("^vt\\s(.+)\\s(.+)");
	regex datoCara_rx ("(\\d+)/(\\d+)/(\\d+)");
	regex repeat_rx("^repeat(.*)");
	regex rot_rx("^rot\\s(.+)\\s(.+)\\s(.+)");
	regex rotend_rx("^rotend.*");
	// Arreglo de matcheo
	smatch matcheo;
	smatch matchCara;
	// Variables 
	map<string, ObjetoGrafico*> mtls;
	Mtl* actualMTL;
	vector<ObjetoGrafico*>* retorno = new vector<ObjetoGrafico*>();
	map<int, float*> vertices;
	map<int, float*> texturas;
	map<int, float*> normales;
	int numVert = 0; // Los elementos en archivos .obj son referenciados a partir de 1
	int numTex = 0;
	int numNorm = 0;
	float** vertCara;
	float** normCara;
	float** texCara;
	float* elemento;
	float datoCara;
	float* posicion = new float[3];
	posicion[0] = posicion[1] = posicion[2] = 0;
	string linea = "";
	string tipo = "";	
	repetir = false;
	// Generar archivo
	ifstream archivo;
	archivo.open(path + nombreObj);
	// Leer lneas e interpretarlas
	while (getline(archivo, linea)) {
		// Parsear linea
		if (regex_match(linea, matcheo, usemtl_rx)) {// Requiere usar una mtl
			if ((matcheo.size() >= 2) && (&(mtls[matcheo[1].str()]) != NULL)) {				
				retorno->push_back(mtls[matcheo[1].str()]);
			}
		}
		else if (regex_match(linea, matcheo, mtllib_rx)) { // Se debe cargar una mtl
			// Cargar mtl
			if (matcheo.size() >= 2) {
				mtls = cargarMTL(path, matcheo[1].str());
			}
		}
		else if (regex_match(linea, matcheo, elemento_rx)) {
			// Ver qu tipo de elemento es
			if (matcheo.size() >= 5) {
				if (matcheo[1].str() == "v") { // Vertice
					// Crear terna
					numVert++;
					elemento = new float[3];
					// x est invertido con z para generar una orientacin de la moto hacia x.
					elemento[0] = stof(matcheo[4]) * mult + posicion[0];
					elemento[1] = stof(matcheo[3]) * mult + posicion[1];
					elemento[2] = stof(matcheo[2]) * mult + posicion[2];
					vertices[numVert] = elemento; // Ingresar terna
				}				
				else if (matcheo[1].str() == "vn") { // Normal
					// Crear terna
					numNorm++;
					elemento = new float[3];
					elemento[0] = stof(matcheo[2]);
					elemento[1] = stof(matcheo[3]);
					elemento[2] = stof(matcheo[4]);
					normales[numNorm] = elemento; // Ingresar terna					
				}
				else if (matcheo[1].str() == "f") { // Cara
					// Se asume que se trabaja con vrtices, texturas y normales ya definidas					
					vertCara = new float*[3];
					normCara = new float*[3];
					texCara = new float*[3];
					for (unsigned i = 2; i < 5; i++) { // Para cada bloque
						bool valido = true;
						string entrada = matcheo[i];
						if (regex_match(entrada, matchCara, datoCara_rx)) { // Encontrar numero de vrtice, textura y normal
							if (matchCara.size() >= 4) {								
								int numV, numT, numN;
								numN = stoi(matchCara[3]);
								numT = stoi(matchCara[2]);
								numV = stoi(matchCara[1]);
								// Buscar vertice, textura y normal
								if ((vertices.size() >= numV) && (texturas.size() >= numT) && (normales.size() >= numN)) {		
									vertCara[i-2] = vertices[numV];
									normCara[i-2] = normales[numN];
									texCara[i-2] = texturas[numT];
								}
							}
						}
					}
					// Crear Cara y agregarla al retorno
					retorno->push_back(new Cara(vertCara,normCara,texCara));
				}
			}
		}
		else if (regex_match(linea, matcheo, tex_rx)) { // Textura
			if (matcheo.size() >= 3) {
				// Crear terna
				numTex++;
				elemento = new float[2];
				elemento[0] = stof(matcheo[1]);
				elemento[1] = stof(matcheo[2]);
				texturas[numTex] = elemento; // Ingresar terna				
			}
		}
		else if (regex_match(linea, matcheo, repeat_rx)) {
			repetir = true;
		}
		else if (regex_match(linea, matcheo, pos_rx)) {
			posicion[0] = stof(matcheo[1]);
			posicion[1] = stof(matcheo[2]);
			posicion[2] = stof(matcheo[3]);
		}
		else if (regex_match(linea, matcheo, rot_rx)) {
			float* centro = new float[3];
			centro[0] = stof(matcheo[1]);
			centro[1] = stof(matcheo[2]);
			centro[2] = stof(matcheo[3]);
			retorno->push_back(new AngularRotation(INICIO,centro));
		}
		else if (regex_match(linea, matcheo, rotend_rx)) {
			retorno->push_back(new AngularRotation(FIN, NULL));
		}
	}
	archivo.close();
	return retorno;
};

vector<ObjetoGrafico*>* parsearVehiculo(string path, string nombreObj, float mult = 1.0) {
	bool repetir = false;
	return parsearObj(path, nombreObj, repetir, mult);
}

#endif
