#pragma once
#ifndef COLLECTIBLEOBJECT_H
#define COLLECTIBLEOBJECT_H

#include "Entidad.h"
#include "Mundo.h"
#include "ConjuntoParticulas.h"
#include "Notificable.h"

#define CANTIDAD_CONJUNTOS_PARTICULAS 10

#include <Box2D/Box2D.h>


//el lado de las cajitas
#define LADO 1.0f

class CollectibleObject : public Entidad{
protected:
	Notificable * nivel;
	ConjuntoParticulas * particulas;
	ObjetoGrafico* mtlLocal;
	int frameCount;
	int particulaCount;
	bool victorioso;
public:
	CollectibleObject(float PosX, float PosY, Notificable * nivel, ObjetoGrafico* mtl) : Entidad() {
		particulas = new ConjuntoParticulas(CANTIDAD_CONJUNTOS_PARTICULAS, CONFETI);
		victorioso = false;
		mtlLocal = mtl;

		posicion.x = PosX;
		posicion.y = PosY;
		this->nivel = nivel;

		b2PolygonShape forma;
		b2Vec2 vertices[4];
		vertices[0].Set(0.0f, 0.0f);
		vertices[1].Set(0.0f, 1.0f);
		vertices[2].Set(1.0f, 1.0f);
		vertices[3].Set(1.0f, 0.0f);
		forma.Set(vertices, 4);

		b2FixtureDef fixtureDef;
		fixtureDef.shape = &forma;
		fixtureDef.isSensor = true;

		bodyDef.position.Set(posicion.x, posicion.y);
		body = mundo.CreateBody(&bodyDef);
		body->CreateFixture(&fixtureDef);

	}

	void actualizar() {
		if (reset) {
			particulas->disableParticulas();
			victorioso = false;
			this->reset = false;
		}
		//logica de los efectos de particulas
		//esto genera una fuente de confeti
		if (victorioso) {
			++frameCount;
			if (particulaCount < CANTIDAD_CONJUNTOS_PARTICULAS) {
				if (frameCount > 60) {
					particulas->nuevoParticulas(posicion.x+LADO/2.0f, posicion.y + LADO / 2.0f, 0.0f);
					++particulaCount;
					frameCount = 0;
				}
			}
			else {
				if (frameCount > 200) {
					particulas->disableParticulas();
					particulas->nuevoParticulas(posicion.x + LADO / 2.0f, posicion.y + LADO / 2.0f, 0.0f);
					particulaCount = 1;
					frameCount = 0;
				}
			}
		}
		particulas->step();
	}

	void notificar(Evento evento, void * arg) {
		switch (evento) {
		case EV_COLISION:
			if (!victorioso) {
				nivel->notificar(EV_VICTORIA, this);
				particulas->nuevoParticulas(posicion.x + LADO / 2.0f, posicion.y + LADO / 2.0f, 0.0f);
				particulaCount = 1;
				frameCount = 0;
				victorioso = true;
			}
			break;
		case EV_VICTORIA:
			break;
		default:
			break;
		}
		
	}

	void dibujar() {
		particulas->dibujar();
		glColor3f(1.0f, 1.0f, 1.0f);
		if (mtlLocal != NULL)
			mtlLocal->dibujar();
		glPushMatrix();
		glTranslatef(posicion.x, posicion.y, 0.0f);
		glBegin(GL_QUADS);

		//cara inferior
		glNormal3f(0.0f, 1.0f, 0.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(0.0f, 0.0f, -LADO/2.0f);
		glTexCoord2f(0.0f, LADO*4.0f);
		glVertex3f(0.0f, 0.0f, LADO/2.0f);
		glTexCoord2f(LADO*4.0f, LADO*4.0f);
		glVertex3f(1.0f, 0.0f, LADO /2.0f);
		glTexCoord2f(LADO*4.0f, 0.0f);
		glVertex3f(1.0f, 0.0f, -LADO /2.0f);

		//cara superior
		glNormal3f(0.0f, -1.0f, 0.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(0.0f, 1.0f, -LADO / 2.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(1.0f, 1.0f, -LADO / 2.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(1.0f, 1.0f, LADO / 2.0f);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(0.0f, 1.0f, LADO / 2.0f);

		//los costados
		glNormal3f(0.0f, 0.0f, 1.0f);
		glTexCoord2f(0.0f, 0.0f);

		glVertex3f(0.0f, 0.0f, LADO/2.0f);
		glTexCoord2f(0.0f, LADO*4.0f);

		glVertex3f(0.0f, LADO, LADO / 2.0f);
		glTexCoord2f(LADO*4.0f, LADO*4.0f);

		glVertex3f(LADO, LADO, LADO / 2.0f);
		glTexCoord2f(LADO*4.0f, 0.0f);

		glVertex3f(LADO, 0.0f, LADO / 2.0f);


		glNormal3f(1.0f, 0.0f, 0.0f);
		glTexCoord2f(0.0f, 0.0f);

		glVertex3f(LADO, 0.0f, LADO / 2.0f);
		glTexCoord2f(0.0f, LADO*4.0f);

		glVertex3f(LADO, LADO, LADO / 2.0f);
		glTexCoord2f(LADO*4.0f, LADO*4.0f);

		glVertex3f(LADO, LADO, -LADO / 2.0f);
		glTexCoord2f(LADO*4.0f, 0.0f);

		glVertex3f(LADO, 0.0f, -LADO / 2.0f);


		glNormal3f(0.0f, 0.0f, -1.0f);
		glTexCoord2f(0.0f, 0.0f);

		glVertex3f(0.0f, 0.0f, -LADO / 2.0f);
		glTexCoord2f(0.0f, LADO*4.0f);

		glVertex3f(0.0f, LADO, -LADO / 2.0f);
		glTexCoord2f(LADO*4.0f, LADO*4.0f);

		glVertex3f(LADO, LADO, -LADO / 2.0f);
		glTexCoord2f(LADO*4.0f, 0.0f);

		glVertex3f(LADO, 0.0f, -LADO / 2.0f);


		glNormal3f(-1.0f, 0.0f, 0.0f);
		glTexCoord2f(0.0f, 0.0f);

		glVertex3f(0, 0.0f, LADO / 2.0f);
		glTexCoord2f(0.0f, LADO*4.0f);

		glVertex3f(0, LADO, LADO / 2.0f);
		glTexCoord2f(LADO*4.0f, LADO*4.0f);

		glVertex3f(0, LADO, -LADO / 2.0f);
		glTexCoord2f(LADO*4.0f, 0.0f);

		glVertex3f(0, 0.0f, -LADO / 2.0f);


		glEnd();
		glPopMatrix();
	}

	~CollectibleObject() {
		delete particulas;
		mundo.DestroyBody(body);
	}
};
#endif //COLLECTIBLEOBJECT_H