#pragma once
#ifndef MAPOBJECT_H
#define MAPOBJECT_H

#include "Entidad.h"
#include "ObjetoGrafico.h"
#include <vector>

//la coordenada z que es el centro del mundo
#define OFFSET 0.0f
//el alto de las cajitas 
#define HEIGHT 0.3f
//la mitad del ancho
#define HWIDTH 2.5f

using namespace std;

class MapObject : public Entidad{
protected:
	ObjetoGrafico* mtlLocal;
	b2ChainShape cadena;
public:
	MapObject(b2Vec2 * puntos, int cantPuntos ,ObjetoGrafico* mtl) : Entidad() {
		bodyDef.position.Set(posicion.x, posicion.y);	//no se si la posicion sirve para algo
		cadena.CreateChain(puntos, cantPuntos);
		b2FixtureDef fixtureDef;
		fixtureDef.shape = &cadena;
		fixtureDef.friction = 1.0f;
		body = mundo.CreateBody(&bodyDef);
		body->CreateFixture(&fixtureDef);
		mtlLocal = mtl;
	}
    void actualizar(float tiempo, string evento){}
	void dibujar() {
		if (mtlLocal != NULL)
			mtlLocal->dibujar();
		glBegin(GL_QUADS);
		for (int32 i = 0; i < cadena.GetChildCount(); ++i) {
			b2EdgeShape segmento;
			cadena.GetChildEdge(&segmento, i);
			{
				float lenx = (segmento.m_vertex2.x - segmento.m_vertex1.x);
				float leny = (segmento.m_vertex2.y - segmento.m_vertex1.y);
				float largo = sqrt(lenx*lenx + leny*leny) / 5.f;
				//cara superior
				glNormal3f(0.0, 1.0, 0.0);
				glTexCoord2f(0.0, 1.0);
				glVertex3f(segmento.m_vertex1.x, segmento.m_vertex1.y, OFFSET + HWIDTH);
				glTexCoord2f(largo, 1.0);
				glVertex3f(segmento.m_vertex2.x, segmento.m_vertex2.y, OFFSET + HWIDTH);
				glTexCoord2f(largo, 0.0);
				glVertex3f(segmento.m_vertex2.x, segmento.m_vertex2.y, OFFSET - HWIDTH);
				glTexCoord2f(0.0, 0.0);
				glVertex3f(segmento.m_vertex1.x, segmento.m_vertex1.y, OFFSET - HWIDTH);

				//cara inferior
				glNormal3f(0.0, -1.0, 0.0);
				glTexCoord2f(0.0, 1.0);
				glVertex3f(segmento.m_vertex1.x, segmento.m_vertex1.y - HEIGHT, OFFSET + HWIDTH);
				glTexCoord2f(largo, 1.0);
				glVertex3f(segmento.m_vertex2.x, segmento.m_vertex2.y - HEIGHT, OFFSET + HWIDTH);
				glTexCoord2f(largo, 0.0);
				glVertex3f(segmento.m_vertex2.x, segmento.m_vertex2.y - HEIGHT, OFFSET - HWIDTH);
				glTexCoord2f(0.0, 0.0);
				glVertex3f(segmento.m_vertex1.x, segmento.m_vertex1.y - HEIGHT, OFFSET - HWIDTH);

				//los costados
				glNormal3f(0.0, 0.0, 1.0);
				glTexCoord2f(0.0, 0.0);
				glVertex3f(segmento.m_vertex1.x, segmento.m_vertex1.y, OFFSET + HWIDTH);
				glTexCoord2f(1.0, 0.0);
				glVertex3f(segmento.m_vertex1.x, segmento.m_vertex1.y - HEIGHT, OFFSET + HWIDTH);
				glTexCoord2f(0.0, 1.0);
				glVertex3f(segmento.m_vertex2.x, segmento.m_vertex2.y - HEIGHT, OFFSET + HWIDTH);
				glTexCoord2f(1.0, 1.0);
				glVertex3f(segmento.m_vertex2.x, segmento.m_vertex2.y, OFFSET + HWIDTH);

				glNormal3f(1.0, 0.0, 0.0);
				glTexCoord2f(0.0, 0.0);
				glVertex3f(segmento.m_vertex2.x, segmento.m_vertex2.y - HEIGHT, OFFSET + HWIDTH);
				glTexCoord2f(1.0, 0.0);
				glVertex3f(segmento.m_vertex2.x, segmento.m_vertex2.y, OFFSET + HWIDTH);
				glTexCoord2f(0.0, 1.0);
				glVertex3f(segmento.m_vertex2.x, segmento.m_vertex2.y - HEIGHT, OFFSET - HWIDTH);
				glTexCoord2f(1.0, 1.0);
				glVertex3f(segmento.m_vertex2.x, segmento.m_vertex2.y, OFFSET - HWIDTH);

				glNormal3f(0.0, 0.0, -1.0);
				glTexCoord2f(0.0, 0.0);
				glVertex3f(segmento.m_vertex2.x, segmento.m_vertex2.y - HEIGHT, OFFSET - HWIDTH);
				glTexCoord2f(1.0, 0.0);
				glVertex3f(segmento.m_vertex2.x, segmento.m_vertex2.y, OFFSET - HWIDTH);
				glTexCoord2f(0.0, 1.0);
				glVertex3f(segmento.m_vertex1.x, segmento.m_vertex1.y, OFFSET - HWIDTH);
				glTexCoord2f(1.0, 1.0);
				glVertex3f(segmento.m_vertex1.x, segmento.m_vertex1.y - HEIGHT, OFFSET - HWIDTH);

				glNormal3f(-1.0, 0.0, 0.0);
				glTexCoord2f(0.0, 0.0);
				glVertex3f(segmento.m_vertex1.x, segmento.m_vertex1.y, OFFSET - HWIDTH);
				glTexCoord2f(1.0, 0.0);
				glVertex3f(segmento.m_vertex1.x, segmento.m_vertex1.y - HEIGHT, OFFSET - HWIDTH);
				glTexCoord2f(0.0, 1.0);
				glVertex3f(segmento.m_vertex1.x, segmento.m_vertex1.y - HEIGHT, OFFSET + HWIDTH);
				glTexCoord2f(1.0, 1.0);
				glVertex3f(segmento.m_vertex1.x, segmento.m_vertex1.y, OFFSET + HWIDTH);
			}
				
		}
		glEnd();
	}
    ~MapObject(){
		mundo.DestroyBody(body);
	}
};

#endif // VEHICULO_H

