
#pragma once
#ifndef MTL_H
#define MTL_H

#include <gl/GLU.h>
#include "ObjetoGrafico.h"
#include "soil\src\SOIL.h"

#include <string>

using namespace std;

/* Las mtl (Material Template Library) describen una textura.
* Las texturas son definidas con la primitiva 'newmtl' tex, donde 'tex' es el nombre de la textura.
* Cada textura se compone de los siguientes atributos:
* - Ambient Color (float,float,float), descripto por 'Ka'
* - Diffuse Color (float,float,float), descripto por 'Kd'
* - Specular Color (float,float,float,float), descripto por 'Ks' y 'Ns'
* - Transparency (float), descripto por 'd' o 'Tr' segn la implementacin.
* - Illumination Model (integer [0..10]), descripto por 'illum'
* - Texture Map (string), descripto por 'map_Kd' en el caso actual.
* Toda textura debe iniciar con newmtl y debe finalizar con map_Kd
*/

class Mtl : public ObjetoGrafico {
	private:
		string nombre;
		float ambientColor[4];
		float diffuseColor[4];
		float specularColor[4];
		float transparency;
		int illum;
		string mapKd;
		GLuint texID;		
	public:
		Mtl(string n,float aC[3], float dC[3], float sC[4], float t, int il, string mKd, string bindPath) : ObjetoGrafico() {
			nombre = n;
			for (int i = 0; i < 4; i++) {
				if (i < 3) {
					ambientColor[i] = aC[i];
					diffuseColor[i] = dC[i];
				}
				specularColor[i] = sC[i];
			}
			transparency = t;
			ambientColor[3] = t;
			diffuseColor[3] = t;
			illum = il;
			mapKd = mKd;
			texID = 0;

			// Atributos de imgen
			int w = 0;
			int h = 0;
			// Utilizacin de SOIL
			unsigned char* datos = SOIL_load_image(bindPath.c_str(), &w, &h, 0, SOIL_LOAD_RGB);
			// Obtener identificador de Textura de openGL
			GLuint textura;
			glGenTextures(1, &textura);
			glBindTexture(GL_TEXTURE_2D, textura);
			// Configurar Textura
			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
			glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
			glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
			glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
			glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE);
			//gluBuild2DMipmaps(GL_TEXTURE_2D, 3, w, h, GL_RGB, GL_UNSIGNED_BYTE, datos);
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, w, h, 0, GL_RGB, GL_UNSIGNED_BYTE, datos);
			delete datos;
			texID = textura;
		}

		void dibujar(float angulodelante = 0.0, float angulotrasero = 0.0) {
			glColor3f(1.0,1.0,1.0);
			// Iluminacin			
			glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,ambientColor);
			glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuseColor);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specularColor);
			// Texturas
			glBindTexture(GL_TEXTURE_2D, texID);
		};
};

#endif
