#ifndef NIVEL_H
#define NIVEL_H

#include <list>
#include <fstream>
#include <string>

#include "Entidad.h"
#include "BackgroundObject.h"
#include "Notificable.h"
#include "CollectibleObject.h"
#include "ContactListener.h"

#include "pugixml-1.7/src/pugixml.hpp"
#include <iostream>

//la coordenada z que es el centro del mundo
#define OFFSET 0.0f
//el alto de las cajitas 
#define HEIGHT 0.3f
//la mitad del ancho
#define HWIDTH 2.5f

using namespace std;

class Nivel: public Notificable {
    private:
		string nombre;
        string directorio;
        vector<Entidad *> entidades;
		Entidad * moto;
		float colorCielo[3];
		//tengo el contactListener asi puedo suscribir a objetos del nivel
		ContactListener* contactListener;
		b2Body* piso;	//el piso de abajo del todo

		bool victorioso;
    public:
        // Crea un nivel en base a los archivos contenidos en el directorio 'directorio'
        Nivel(string dir, ContactListener* contactListener, Entidad* moto){
			this->contactListener = contactListener;
			this->moto = moto;

			map<string, ObjetoGrafico*> mtls;
			colorCielo[0] = colorCielo[1] = colorCielo[2] = 1.0;
			pugi::xml_document doc;
			directorio = dir;
			pugi::xml_parse_result result = doc.load_file(directorio.c_str());

			pugi::xml_node nivel = doc.child("nivel");
			nombre = nivel.attribute("nombre").value();
			
			pugi::xml_node cielo = nivel.child("cielo");
			colorCielo[0] = stof(string(cielo.attribute("x").value()));
			colorCielo[1] = stof(string(cielo.attribute("y").value()));
			colorCielo[2] = stof(string(cielo.attribute("z").value()));

			for (pugi::xml_node_iterator it = nivel.begin(); it != nivel.end(); it++) {
				if (string(it->name()) == "mtl") {
					mtls = cargarMTL(it->attribute("path").value(), it->attribute("nombreObj").value());
				}
				else if (string(it->name()) == "mapObject") {
					ObjetoGrafico* mtlEnvio = NULL;
					vector<b2Vec2> puntos;
					pugi::xml_node xml_mapObj = *it;
					for (pugi::xml_node_iterator itMap= xml_mapObj.begin(); itMap != xml_mapObj.end(); itMap++) {
						if ((string(itMap->name())) == "usemtl") {
							// Ingresar mtl a objetos
							mtlEnvio = mtls[itMap->attribute("name").value()];
						}
						else if ((string(itMap->name())) == "punto") {
							float xAct = stof(itMap->attribute("x").value());
							float yAct = stof(itMap->attribute("y").value());
							b2Vec2 coords(xAct, yAct); // Puede que haya problemas con esto por casting automtico
							puntos.push_back(coords);
						}
					}
					MapObject * mapObj = new MapObject(puntos.data(), puntos.size(), mtlEnvio);
					entidades.push_back(mapObj);
				}
				else if (string(it->name()) == "backgroundObject") {
					Entidad* test = new BackgroundObject(it->attribute("path").value(), it->attribute("nombreObj").value());
					entidades.push_back(test);
				}
				else if (string(it->name()) == "collectibleObject") {
					//hardcodeo la textura
					ObjetoGrafico* mtlEnvio;
					try {
						mtlEnvio = mtls["checkered"];
					}
					catch (exception e) {
						//forgive me father for I have sinned
						//esto solo deberia pasar si cargan un mtl que no tiene checkered
						mtlEnvio = NULL;
					}
					float xAct = stof(it->attribute("x").value());
					float yAct = stof(it->attribute("y").value());
					Entidad * nuevoObjeto = new CollectibleObject(xAct,yAct,this, mtlEnvio);
					contactListener->Suscribir(nuevoObjeto);
					entidades.push_back(nuevoObjeto);
				}
			}

			//seteo el color del cielo
			glClearColor(colorCielo[0], colorCielo[1], colorCielo[2], 1);
			//indica si estoy en condicion de victoria
			victorioso = false;

			//creo el piso
			//arbitrariamente va entre -50 y 1000
			b2Vec2 v1(-50.0f, -15.0f);
			b2Vec2 v2(1000.0f, -15.0f);
			b2EdgeShape edge;
			edge.Set(v1, v2);
			b2FixtureDef fixtureDef;
			fixtureDef.shape = &edge;
			fixtureDef.friction = 0.3f;
			b2BodyDef bodyDef;
			piso = mundo.CreateBody(&bodyDef);
			piso->CreateFixture(&fixtureDef);
        };

		float* getCielo() { return colorCielo; };

        void dibujar(){
			if (victorioso) {
				//hacer algo
			}
			for (int i = 0; i < entidades.size(); ++i)
				entidades[i]->dibujar();
        };

		void actualizar() {
			for (int i = 0; i < entidades.size(); ++i)
				entidades[i]->actualizar();
		}

		//notificar por algun evento (una victoria)
		void notificar(Evento evento, void * arg) {
			switch (evento){
			case EV_COLISION:
				break;
			case EV_VICTORIA:
			{
				CollectibleObject * collObj = static_cast<CollectibleObject*>(arg);
				//indicar victoria
				victorioso = true;
				moto->notificar(EV_VICTORIA, NULL);
				break;
			}
			default:
				break;
			}
		}

		//resetear a las posiciones iniciales
		void resetNextAct() {
			victorioso = false;
			for (int i = 0; i < entidades.size(); ++i)
				entidades[i]->resetNextAct();
		}

		//Pedir el nombre del nivel
		string getNombre() {
			return this->nombre + "  ";
		}

		bool esVictorioso() {
			return victorioso;
		}

        ~Nivel(){
			contactListener->Desuscribir();
			mundo.DestroyBody(piso);
			for (int i = 0; i < entidades.size(); ++i)
				delete entidades[i];
            entidades.clear();
        };
};

#endif // NIVEL_H
