#pragma once
#ifndef PARTICULAS_H
#define PARTICULAS_H

#include <cstdlib>

#include "SDL.h"
#include "SDL_opengl.h"

#include "Vector_Particula.h"

#define CANTIDAD_PARTICULAS 4000
#define LADO_PARTICULA 0.02

enum Modo
{
	EXPLOSION,
	CONFETI
};

//Esta clase se usa para definir un conjunto de particulas que salen volando de un punto
//Para usarla, ponerla como compuesta en alguna otra clase (por ejemplo vehiculo)
//Le decis a partir de que punto salen volando y despues la haces avanzar step a step, y las dibujas en cada paso
//Idealmente usar el tamano del step del mundo
class Particulas {
private:
	int cantidad_particulas;
	//el modo que indica el estilo
	//de empezar a producirse estilos muy dispares podria hacer varias clases
	Modo modo;
	//TODO podria hacer que las particulas y las velocidades esten en el mismo array con un struct que sea dupla o algo asi
	//para el cache locality
	//igual creo que asi no deberia generar problemas
	vect * particulas;
	vect * velocidades;
	//los colores de las particulas
	GLfloat * coloresR;
	GLfloat * coloresG;
	GLfloat * coloresB;
	//la aceleracion de la simulacion
	vect acel;

	//la posicion en coordenadas del mundo (la uso para dibujar)
	vect posicion;
	//el color de las particulas
	//TODO cada particula un color distinto
	GLfloat color[3] = { 1.0f, 0.0f, 0.0f };

	//TODO hacer que lo pasen en el constructor
	float timeStep = 8.0f / 1000.0f;


	//retorna un float entre [min,max]
	//no garantizado que max y min esten incluidos
	static inline float randFloat(float min, float max) {
		return min + static_cast <float> (rand()) / (static_cast <float> (RAND_MAX / (max - min)));
	}

	//retorna un float entre [-max,-min]U[min,max]
	//no garantizado que max y min esten incluidos
	static inline float randSignedFloat(float min, float max) {
		if ((rand() % 2) == 0)
			return randFloat(min, max);
		else
			return - randFloat(min, max);
	}

public:
	Particulas(int cantidad) {
		cantidad_particulas = cantidad;
		this->modo = EXPLOSION;
		particulas = new vect[cantidad_particulas];
		velocidades = new vect[cantidad_particulas];
		coloresR = new GLfloat[cantidad_particulas];
		coloresG = new GLfloat[cantidad_particulas];
		coloresB = new GLfloat[cantidad_particulas];

		vect acel(0.0f, -10.0f, 0.0f);
		acel.multiplicar(timeStep*timeStep);	//uso distancias para todo, de forma de hacer la menor cantidad de operaciones posible
		this->acel = acel;
	}
	Particulas() : Particulas(CANTIDAD_PARTICULAS) {

	}
	void setModo(Modo modo) {
		this->modo = modo;
	}
	~Particulas() {
		delete[] particulas, cantidad_particulas;	//creo que se hace asi
		delete[] velocidades, cantidad_particulas;
	}

	//Cuando hay una nueva explosion, usar esto
	void setInicio(float x, float y, float z) {
		posicion.setPos(x, y, z);
		for (int i = 0; i < cantidad_particulas; ++i) {
			particulas[i].resetPos();
			velocidades[i].x = randSignedFloat(0.0f, 10.0f);
			velocidades[i].z = randSignedFloat(0.0f, 10.0f);
			velocidades[i].y = randFloat(2.0f, 10.0f);
			velocidades[i].normalizar();
			velocidades[i].multiplicar(randFloat(0.1f, 10.0f));
			velocidades[i].multiplicar(timeStep);	//uso distancias para todo, de forma de hacer la menor cantidad de operaciones posible
			switch (modo) {
			case EXPLOSION:
				coloresR[i] = randFloat(0.8f, 1.0f);
				coloresG[i] = randFloat(0.0f, 0.5f);
				coloresB[i] = 0.0f;
				break;
			case CONFETI:
				coloresR[i] = randFloat(0.1f, 1.0f);
				coloresG[i] = randFloat(0.1f, 1.0f);
				coloresB[i] = randFloat(0.1f, 1.0f);
				break;
			default:
				coloresR[i] = 0.0f;
				coloresG[i] = 0.0f;
				coloresB[i] = 0.0f;
			}

		}
	}
	void step() {
		for (int i = 0; i < cantidad_particulas; ++i) {
			particulas[i].sumar(velocidades[i]); //Pn+1 = Pn + Vn*dt
			velocidades[i].sumar(acel);			//Vn+1 * dt = Vn *dt + g * dt^2
		}
	}
	//dibuja en torno a la posicion que se le paso al construir
	void dibujar() {
		glPushMatrix();
		glTranslatef(posicion.x, posicion.y, posicion.z);
		glColor3fv(color);
		glDisable(GL_TEXTURE_2D);
		glEnable(GL_COLOR_MATERIAL);
		glBegin(GL_QUADS);
		for (int i = 0; i < cantidad_particulas; ++i) {
			
			glColor3f(coloresR[i],coloresG[i],coloresB[i]);

			glVertex3f(particulas[i].x, particulas[i].y, particulas[i].z);
			glVertex3f(particulas[i].x + LADO_PARTICULA, particulas[i].y, particulas[i].z);
			glVertex3f(particulas[i].x + LADO_PARTICULA, particulas[i].y + LADO_PARTICULA, particulas[i].z);
			glVertex3f(particulas[i].x, particulas[i].y + LADO_PARTICULA, particulas[i].z);

			glVertex3f(particulas[i].x, particulas[i].y, particulas[i].z);
			glVertex3f(particulas[i].x, particulas[i].y, particulas[i].z + LADO_PARTICULA);
			glVertex3f(particulas[i].x, particulas[i].y + LADO_PARTICULA, particulas[i].z + LADO_PARTICULA);
			glVertex3f(particulas[i].x, particulas[i].y + LADO_PARTICULA, particulas[i].z);

			glVertex3f(particulas[i].x, particulas[i].y, particulas[i].z);
			glVertex3f(particulas[i].x + LADO_PARTICULA, particulas[i].y, particulas[i].z);
			glVertex3f(particulas[i].x + LADO_PARTICULA, particulas[i].y, particulas[i].z + LADO_PARTICULA);
			glVertex3f(particulas[i].x, particulas[i].y, particulas[i].z + LADO_PARTICULA);
		}
		glEnd();
		glDisable(GL_COLOR_MATERIAL);
		if (texturasHabilitadas) 
			glEnable(GL_TEXTURE_2D);
		glPopMatrix();
	}
};

#endif //PARTICULAS_H