#pragma once
#ifndef VEHICULO_H
#define VEHICULO_H

#include "Entidad.h"
#include "ConjuntoParticulas.h"
#include "ObjetoGrafico.h"

#define CANTIDAD_CONJUNTOS_PARTICULAS 10

#define MAX_MOTOR_SPEED 80.0f
#define TORQUE_RUEDA 10.0f
#define TORQUE_GIRO 23.0f
#define POS_INI_X 0.0f
#define POS_INI_Y 5.0f

using namespace std;

enum estado{NO_ACELERANDO, ACELERANDO_CW, ACELERANDO_CCW};

class Vehiculo : public Entidad {
protected:
	//los elementos de la moto
	b2Body * rueda_delante;
	b2Body * rueda_trasera;
	b2WheelJoint * amort_delante;
	b2WheelJoint * amort_trasero;
	//para la aceleracion
	estado estado_rueda_trasera;
	estado estado_giro;

	//bandera de input deshabilitado
	bool input_enabled;
	int muertes;

	ConjuntoParticulas * particulas;

	void init() {
		muertes = 0;
		particulas = new ConjuntoParticulas(CANTIDAD_CONJUNTOS_PARTICULAS, EXPLOSION);

		posicion.x = POS_INI_X;
		posicion.y = POS_INI_Y;

		estado_rueda_trasera = NO_ACELERANDO;
		estado_giro = NO_ACELERANDO;
		this->reset = false;
		input_enabled = true;

		bodyDef.type = b2_dynamicBody;	//es de tipo dinamico
		bodyDef.position.Set(posicion.x, posicion.y + 1.0f);


		//los tamanos estan hardcodeados, se corresponden con el modelo
		b2PolygonShape chasis;
		b2Vec2 vertices[4];
		vertices[0].Set(-1.5f, 0.0f);
		vertices[1].Set(1.5f, 0.0f);
		vertices[2].Set(1.0f, 0.4f);
		vertices[3].Set(-1.0f, 0.4f);
		chasis.Set(vertices, 4);

		b2CircleShape circulo;
		circulo.m_radius = 0.55f;

		//creo el cuerpo
		b2FixtureDef fixtureDef;
		fixtureDef.shape = &chasis;
		fixtureDef.density = 0.5f;			//la densidad del chasis
		fixtureDef.friction = 1.0f;
		fixtureDef.restitution = 0.0f;

		body = mundo.CreateBody(&bodyDef);
		body->CreateFixture(&fixtureDef);

		//creo las ruedas
		fixtureDef.density = 1.0f;			//la densidad de las ruedas
		fixtureDef.shape = &circulo;
		bodyDef.angularDamping = 0.3f;		//esto decide que tanto se frena el vehiculo cuando se esta moviendo por inercia
		//rueda trasera
		bodyDef.position.Set(posicion.x - 1.3f, posicion.y);		//si cambias estas posiciones cambialas tambien en el reset
		rueda_trasera = mundo.CreateBody(&bodyDef);
		rueda_trasera->CreateFixture(&fixtureDef);
		//rueda delantera
		bodyDef.position.Set(posicion.x + 1.3f, posicion.y);
		rueda_delante = mundo.CreateBody(&bodyDef);
		rueda_delante->CreateFixture(&fixtureDef);

		//creo los joints
		b2WheelJointDef jointDef;
		b2Vec2 eje(0.0f, 1.0f);

		jointDef.Initialize(body, rueda_trasera, rueda_trasera->GetPosition(), eje);
		//ir probando cambiar estos valores para obtener la jugabilidad optima
		jointDef.motorSpeed = 0.0f;
		jointDef.maxMotorTorque = TORQUE_RUEDA;
		jointDef.enableMotor = false;
		jointDef.frequencyHz = 10.0f;
		jointDef.dampingRatio = 0.7f;
		amort_trasero = (b2WheelJoint*)mundo.CreateJoint(&jointDef);

		jointDef.Initialize(body, rueda_delante, rueda_delante->GetPosition(), eje);
		//ir probando cambiar estos valores para obtener la jugabilidad optima
		jointDef.motorSpeed = 0.0f;
		jointDef.maxMotorTorque = 0.0f;
		jointDef.enableMotor = false;	//solo la rueda trasera tiene traccion
		jointDef.frequencyHz = 10.0f;
		jointDef.dampingRatio = 0.7f;
		amort_delante = (b2WheelJoint*)mundo.CreateJoint(&jointDef);
	}

public:	
	Vehiculo(vector<ObjetoGrafico*> ternas) {
		datosObj = ternas;
		init();
		
	};
	void actualizar() {
		if (reset) {
			estado_rueda_trasera = NO_ACELERANDO;
			estado_giro = NO_ACELERANDO;

			body->SetTransform(b2Vec2(POS_INI_X, POS_INI_Y), 0.0f);
			body->SetAngularVelocity(0.0f);
			body->SetLinearVelocity(b2Vec2_zero);

			rueda_trasera->SetTransform(b2Vec2(POS_INI_X - 1.3f, POS_INI_Y), 0.0f);
			rueda_trasera->SetAngularVelocity(0.0f);
			rueda_trasera->SetLinearVelocity(b2Vec2_zero);

			rueda_delante->SetTransform(b2Vec2(POS_INI_X + 1.3f, POS_INI_Y), 0.0f);
			rueda_delante->SetAngularVelocity(0.0f);
			rueda_delante->SetLinearVelocity(b2Vec2_zero);

			input_enabled = true;
			particulas->disableParticulas();
			this->reset = false;
		}

		//actualizo posicion de particulas
		particulas->step();

		//logica de movimiento del vehiculo
		switch (estado_rueda_trasera) {
		case NO_ACELERANDO:
			if (amort_trasero->IsMotorEnabled()) {
				amort_trasero->EnableMotor(false);
				amort_trasero->SetMotorSpeed(0.0f);
			}
			break;
		case ACELERANDO_CW:
			if (!amort_trasero->IsMotorEnabled()) {
				amort_trasero->EnableMotor(true);
				amort_trasero->SetMotorSpeed(rueda_trasera->GetAngularVelocity());
			}
			else if (amort_trasero->GetMotorSpeed() > 0) {	//si esta yendo en reversa
				amort_trasero->SetMotorSpeed(0.0f);
			}
			else if (amort_trasero->GetMotorSpeed() > -MAX_MOTOR_SPEED) {	//top speed de la rueda
				amort_trasero->SetMotorSpeed(amort_trasero->GetMotorSpeed() - 0.07f);
			}
			body->ApplyTorque(-3.0f, true);	//hace que sea mas dificil que se despegue del piso cuando acelera
			break;
		case ACELERANDO_CCW:
			if (!amort_trasero->IsMotorEnabled()) {
				amort_trasero->EnableMotor(true);
				amort_trasero->SetMotorSpeed(0.0f);
			}
			else if (amort_trasero->GetMotorSpeed() < 20.0f) {	//top speed de la rueda marcha atras
				amort_trasero->SetMotorSpeed(amort_trasero->GetMotorSpeed() + 0.3f);
			}
			break;
		}
		switch (estado_giro) {
		case NO_ACELERANDO:
			break;
		case ACELERANDO_CW:
			body->ApplyTorque(-TORQUE_GIRO, true);	//la fuerza de giro
			break;
		case ACELERANDO_CCW:
			body->ApplyTorque(TORQUE_GIRO, true);
			break;
		}
	};
	void down_key(SDL_Keycode tecla) {
		if (input_enabled) {
			switch (tecla) {
			case SDLK_UP:
				estado_rueda_trasera = ACELERANDO_CW;
				break;
			case SDLK_DOWN:
				estado_rueda_trasera = ACELERANDO_CCW;
				break;
			case SDLK_LEFT:
				estado_giro = ACELERANDO_CCW;
				break;
			case SDLK_RIGHT:
				estado_giro = ACELERANDO_CW;
				break;
			default:
				break;
			}
		}
	}
	void up_key(SDL_Keycode tecla) {
			switch (tecla) {
			case SDLK_UP:
			case SDLK_DOWN:
				estado_rueda_trasera = NO_ACELERANDO;
				break;
			case SDLK_LEFT:
			case SDLK_RIGHT:				
				estado_giro = NO_ACELERANDO;
				break;
			default:
				break;
			}
	}

	void notificar(Evento evento, void * arg) {
		switch (evento) {
		case EV_COLISION:
		{
			//en caso de colision arg es el punto de contacto
			b2Vec2* punto_contacto = static_cast<b2Vec2*> (arg);
			if (input_enabled) {
				++muertes;
				input_enabled = false;
			}
			estado_giro = NO_ACELERANDO;
			estado_rueda_trasera = NO_ACELERANDO;

			particulas->nuevoParticulas(punto_contacto->x, punto_contacto->y, 0.0f);
			break;
		}
		case EV_VICTORIA:
			input_enabled = false;
			estado_giro = NO_ACELERANDO;
			estado_rueda_trasera = NO_ACELERANDO;
			break;
		default:
			break;
		}
		
	}

	void dibujar() {

		particulas->dibujar();
		
		float32 velrueda = rueda_trasera->GetAngularVelocity();
		b2Vec2 position = body->GetPosition();
		float32 angle_rad = body->GetAngle();
		glPushMatrix();
		glTranslatef(position.x, position.y, 0.0f);
		glRotatef(RadtoDeg(angle_rad), 0.0f, 0.0f, 1.0f);
		glPushMatrix();
		glTranslatef(0.0, -1.55, 0.0);			
		for (vector<ObjetoGrafico*>::iterator it = datosObj.begin(); it < datosObj.end(); it++)
			(*it)->dibujar(RadtoDeg(rueda_delante->GetAngle()), RadtoDeg(rueda_trasera->GetAngle()));
		glPopMatrix();
		glPopMatrix();
	};
	
	int getMuertes() {
		return muertes;
	}

	~Vehiculo() {
		delete particulas;
		mundo.DestroyBody(body);
		mundo.DestroyBody(rueda_delante);
		mundo.DestroyBody(rueda_trasera);
	};
};

#endif // VEHICULO_H
