#include <iostream>
#include "SDL2/SDL.h"
#include "SDL2/SDL_opengl.h"
#include "SDL2/SDL_ttf.h"
#undef main
#include <GL/glu.h>
#include <fstream>
#include <glm/glm.hpp>
#include "FreeImage/FreeImage.h"
#include "Mapa.h"
#include "Moto.h"
#include "Menu.h"
#include "HUD.h"
#include "ObjLoader.h"

#define FPS 60
using namespace std;


void actualizarCam(float &x,float &y, float &z, float x_angle, float y_angle,float radius, float motoX, float motoY, float motoZ){
    z = motoZ+cos(y_angle*M_PI/180) * cos(x_angle*M_PI/180) * radius;
    x = motoX+sin(y_angle*M_PI/180) * cos(x_angle*M_PI/180) * radius;
    y = motoY+sin(x_angle*M_PI/180) * radius;
}

void DrawQuadM(int x, int y, int w, int h){
    glDisable(GL_LIGHTING);
    glBegin(GL_QUADS);
        glTexCoord2d(0,0);
        glVertex2d(x,y);
        glTexCoord2d(1,0);
        glVertex2d(x+w,y);
        glTexCoord2d(1,1);
        glVertex2d(x+w,y+h);
        glTexCoord2d(0,1);
        glVertex2d(x,y+h);
    glEnd();
    glEnable(GL_LIGHTING);
}


void init2dM(int w, int h){
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0, w, 0, h, -1, 1);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

void init3dM(bool wireframe, bool textures){
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45, 640/480.f, 0.1, 100);
    glEnable(GL_DEPTH_TEST);
    glMatrixMode(GL_MODELVIEW);
    if(textures)
        glEnable(GL_TEXTURE_2D);
    else
        glDisable(GL_TEXTURE_2D);
}

void setLighting(GLfloat* ambient, GLfloat* diffuse, GLfloat* specular, GLfloat* position) {
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
    glLightfv(GL_LIGHT0, GL_SPECULAR, specular);
    glLightfv(GL_LIGHT0, GL_POSITION, position);
};

SDL_Window *window;
SDL_GLContext windowContext;

bool init() {
    if(SDL_Init(SDL_INIT_VIDEO)<0) {
        cerr << "No se pudo iniciar SDL: " << SDL_GetError() << endl;
        return false;
    }

    window = SDL_CreateWindow("My Game Window",SDL_WINDOWPOS_CENTERED,SDL_WINDOWPOS_CENTERED,1280, 720,SDL_WINDOW_OPENGL);
    if(!window){
        cerr << "No se pudo iniciar SDL_Window: " << SDL_GetError() << endl;
        return false;
    }

    windowContext = SDL_GL_CreateContext(window);
    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
    SDL_GL_SetSwapInterval(1);

    TTF_Init();

    //Imagen de carga

    glClearColor(1.0,1.0,1.0,1.0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    int w,h;
    SDL_GetWindowSize(window,&w,&h);
    init2dM(w,h);
    TexLoader loading;
    loading.Load("Texturas/loading.jpg", GL_RGB);
    glBindTexture(GL_TEXTURE_2D, loading.GetID());
    glEnable(GL_TEXTURE_2D);
    glPushMatrix();
    glBegin(GL_QUADS);
        glTexCoord2d(0,0);  glVertex2d(0,0);
        glTexCoord2d(1,0);  glVertex2d(w,0);
        glTexCoord2d(1,1);  glVertex2d(w,h);
        glTexCoord2d(0,1);  glVertex2d(0,h);
    glEnd();
    glBindTexture(GL_TEXTURE_2D,0);
    glPopMatrix();
    glDisable(GL_TEXTURE_2D);
    SDL_GL_SwapWindow(window);
    init3dM(false,true);

    return true;
}

void close(){
    TTF_Quit();
    SDL_DestroyWindow(window);
    SDL_Quit();
}

void init2D(){
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0, 360, 0, 360, -1, 1);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}



int main(){

    if(!init())
        return 0;

    int w,h;
    SDL_GetWindowSize(window,&w,&h);
    //Movimiento camara.
    float anga=0;
    float angb=0;
    float rad=-10;

    //Posicion Moto.
    float motoX=0;
    float motoY=0;
    float motoZ=0;

    //Settings flags.
    bool moverCam=false;
    bool pausa=false;
    bool fin=false;
    bool wasted=false;
    bool win= false;
    bool wireframeOn=false;
    bool texturesOn=true;
    float gameSpeed= 1.0;
    bool fullscreen=false;
    int nivel= 1;
    int speed=0;
    int points=0;

    //POV
    float x=-10;
    float y=10;
    float z=0;

    Mapa mapa;
    Moto moto;
    Menu menu;
    HUD hud;

    mapa.Load(1);
    hud.Load();

    //Manejo de la luz
    int skyboxID = 0;

    //luz
    GLfloat ambient[] = { .7, .7, .7, 1.0 };
    GLfloat diffuse[] = { .3, .3, .3, 1.0 };
    GLfloat specular[] = { 1.0, 1.0, 1.0, 1.0 };
    GLfloat position[] = { 0.0, 10.0, 0.0, 0.0 };

    glEnable(GL_DEPTH_TEST);

    //Seleccion de moto
    vector<Moto*> motos;
    ObjLoader objL;
    Moto* m1= new Moto();
    Moto* m2= new Moto();
    Moto* m3= new Moto();
    m1->Load(1);
    m2->Load(2);
    m3->Load(3);
    motos.push_back(m1);
    motos.push_back(m2);
    motos.push_back(m3);
    int indice;
    menu.init();
    menu.runInicio(window, motos, indice, fin);
    if(fin)
        return 0;
    for(int i=3; i>=0; i--){
        if(i==indice)
            moto= *motos[indice];
        else
            delete(motos[i]);
        motos.pop_back();
    }
    moto.setMapa(&mapa);


    //argumentos para controlar la cantidad de frames
    float deltaTime = 0.0;
    int thisTime = 0;
    int lastTime = 0;

    //argumentos para movimiento de la moto
    float incline = 0;
    float aceleracion =0;


    //Mensaje de nivel
    TexLoader niv1, niv2;
    niv1.Load("Texturas/nivel1.png", GL_RGBA);
    niv2.Load("Texturas/nivel2.png", GL_RGBA);
    vector<TexLoader> cartelNiveles;
    cartelNiveles.push_back(niv1);
    cartelNiveles.push_back(niv2);
    float tStart=0;


    SDL_Event evento;
    while(!fin){
        thisTime = SDL_GetTicks();
        deltaTime = (float)(thisTime - lastTime) / 1000;
        lastTime = thisTime;

        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        glLoadIdentity();
        gluLookAt(x,y+5,-z,motoX,motoY,motoZ,0,1,0);

        setLighting(ambient, diffuse, specular, position);

        init3dM(wireframeOn, texturesOn);
        mapa.Draw(x, y, -z, skyboxID);
        moto.mover(aceleracion, incline, motoX, motoY ,gameSpeed, deltaTime,wasted, win, points);
        actualizarCam(x,y,z,angb,anga,rad,motoX,motoY,motoZ);
        moto.Draw();

        //Cartel de nivel
        if(tStart<1.0){
            tStart+= 0.02;
            init2dM(w,h);
            glEnable(GL_TEXTURE_2D);
            glEnable(GL_BLEND);
            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
            glBindTexture(GL_TEXTURE_2D, cartelNiveles[nivel-1].GetID());
            glPushMatrix();
            DrawQuadM(w/2 - 128, h/2-64, 256, 128);
            glPopMatrix();
            glDisable(GL_BLEND);
            glDisable(GL_TEXTURE_2D);
        }

        //HUD
        init2dM(w, h);
        glDisable(GL_DEPTH_TEST);
        glDisable(GL_LIGHTING);
        hud.Draw(speed, points);
        glEnable(GL_LIGHTING);
        glEnable(GL_DEPTH_TEST);

        int xm,ym;
        SDL_GetMouseState(&xm, &ym);

        while(SDL_PollEvent(&evento)){
            switch(evento.type){
                case SDL_MOUSEBUTTONDOWN:
                    if(SDL_GetMouseState(NULL, NULL)&SDL_BUTTON(3)){
                        moverCam=true;
                    }
                    break;
                case SDL_MOUSEBUTTONUP:
                    moverCam=false;
                    break;
                case SDL_MOUSEMOTION:
                    if(moverCam){
                        if (evento.motion.yrel<0 && angb < 80 )
                            angb-=evento.motion.yrel*0.4;//factor de ajuste: 0,4
                        else if (evento.motion.yrel>=0 && angb > -80)
                            angb-=evento.motion.yrel*0.4;//factor de ajuste: 0,4
                        anga+=evento.motion.xrel*0.4;//factor de ajuste: 0,4
                        actualizarCam(x,y,z,angb,anga,rad,motoX,motoY,motoZ);
                    }
                    break;
                case SDL_QUIT:
                    fin = true;
                    break;
                case SDL_KEYDOWN:
                    switch(evento.key.keysym.sym){
                        case SDLK_ESCAPE:
                            fin = true;
                            break;
                        case SDLK_LEFT:
                            incline = 0.7;
                        break;
                        case SDLK_RIGHT:
                            incline = -0.5;
                        break;
                        case SDLK_UP:
                            aceleracion = 20;
                            speed = 1;
                            break;
                        case SDLK_DOWN:
                            aceleracion = -10;
                            speed = 1;
                        break;
                        case SDLK_p:
                            menu.setCamara(x,y+5,z,motoX, motoY, motoZ);
                            menu.runPausa(window, &mapa, &moto, wireframeOn, texturesOn, fin, fullscreen);
                            if(!fin){
                                if(fullscreen){
                                    SDL_SetWindowSize(window, 1280, 720);
                                    SDL_SetWindowPosition(window, SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED);
                                }
                                else{
                                    SDL_SetWindowSize(window, 1280, 720);
                                    SDL_SetWindowPosition(window, SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED);
                                }
                                if(wireframeOn)
                                    glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
                                else
                                    glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
                            }
                            lastTime = SDL_GetTicks();
                            break;
                        case SDLK_r:
                            moto.restart();
                            tStart= 0;
                            points= 0;
                            wasted = false;
                            break;
                        case SDLK_s:
                            rad-=.05;//factor de ajuste: 0,05
                            actualizarCam(x,y,z,angb,anga,rad,motoX,motoY,motoZ);
                            break;
                        case SDLK_w:
                            if(rad<0)
                                rad+=.05;//factor de ajuste: 0,05
                            actualizarCam(x,y,z,angb,anga,rad,motoX,motoY,motoZ);
                            break;
                        case SDLK_m:
                            gameSpeed+=0.1;
                            break;
                        case SDLK_n:
                            if (gameSpeed>0)
                                gameSpeed-=0.1;
                            break;
                        case SDLK_c:
                            skyboxID = 0;
                            ambient[0] = .6;ambient[1] = .6;ambient[2] = .6;ambient[3] = 1.0;
                            diffuse[0] = .4;diffuse[1] = .4;diffuse[2] = .4;diffuse[3] = 1.0;
                            specular[0] = 1.0;specular[1] = .0;specular[2] = .0;specular[3] = 1.0;
                            position[0] = 0.0;position[1] = 10.0;position[2] = 0.0;position[3] = 1.0;
                            break;
                        case SDLK_z:
                            ambient[0] = .1;ambient[1] = .1;ambient[2] = .1;ambient[3] = 1.0;
                            diffuse[0] = .3;diffuse[1] = .2;diffuse[2] = .0;diffuse[3] = 1.0;
                            specular[0] = 1.0;specular[1] = .0;specular[2] = .0;specular[3] = 1.0;
                            position[0] = 10.0;position[1] = 10.0;position[2] = 10.0;position[3] = 1.0;
                            skyboxID = 1;
                            break;
                        case SDLK_x:
                            ambient[0] = .1;ambient[1] = .1;ambient[2] = .2;ambient[3] = 1.0;
                            diffuse[0] = .2;diffuse[1] = .3;diffuse[2] = .5;diffuse[3] = 1.0;
                            specular[0] = 1.0;specular[1] = 1.0;specular[2] = 1.0;specular[3] = 1.0;
                            position[0] = -10.0;position[1] = -10.0;position[2] = -10.0;position[3] = 1.0;
                            skyboxID = 2;
                            break;
                    }
                    break;
                case SDL_KEYUP:
                    switch(evento.key.keysym.sym){
                        case SDLK_LEFT:
                            incline = 0;
                        break;
                        case SDLK_RIGHT:
                            incline = 0;
                        break;
                        case SDLK_UP:
                            aceleracion = 0;
                            speed = 0;
                            break;
                        case SDLK_DOWN:
                            aceleracion = 0;
                            speed = 0;
                            break;
                        case SDLK_h:
                            moto.switchHumo();
                            break;
                        case SDLK_j:
                            moto.switchBillboard();
                            break;
                    }
                    break;
            }
        }
        if (1000.0/FPS > deltaTime)
            SDL_Delay(1000.0/FPS - deltaTime);
        SDL_GL_SwapWindow(window);

        if(win){
            mapa.Load(2);
            moto.setMapa(&mapa);
            moto.restart();
            tStart= 0;
            points= 0;
            wasted= false;
            nivel= 2;
            incline = 0;
            aceleracion = 0;
        }

        if(wasted){
            menu.setCamara(x,y+5,z,motoX, motoY, motoZ);
            menu.runWasted(window, wasted, &mapa, &moto, wireframeOn, texturesOn, fin, fullscreen, points);
            moto.restart();
            tStart= 0;
            points= 0;
            incline = 0;
            aceleracion = 0;
        }
    };

    close();



    return 0;
}
