#include <iostream>
#include "Menu.h"
#include "SDL2/SDL.h"
#include "SDL2/SDL_opengl.h"
#include <GL/glu.h>
#define margin_side w/10
#define margin_bot h/10
#include "ObjLoader.h"
#define FPS 60

Menu::Menu()
{
}

Menu::~Menu()
{
    //dtor
}

bool Menu::init(){
    bool ok= true;
    ok= ok && id_wireframe.Load("Texturas/wireframe.png", GL_RGBA);
    ok= ok && id_texturas.Load("Texturas/texturas.png", GL_RGBA);
    ok= ok && id_velocidad.Load("Texturas/velocidad.png", GL_RGBA);
    ok= ok && id_fullscreen.Load("Texturas/fullscreen.png", GL_RGBA);
    ok= ok && id_numeros.Load("Texturas/numeros.png", GL_RGBA);
    ok= ok && id_tick.Load("Texturas/tick.png", GL_RGBA);
    ok= ok && id_notick.Load("Texturas/notick.png", GL_RGBA);
    ok= ok && id_pausa.Load("Texturas/pausa.png", GL_RGBA);
    ok= ok && id_comenzar.Load("Texturas/comenzar.jpg", GL_RGB);
    ok= ok && id_flecha.Load("Texturas/flecha.jpg", GL_RGB);
    ok= ok && id_reintentar.Load("Texturas/reintentar.png", GL_RGBA);
    ok= ok && id_salir.Load("Texturas/salir.png", GL_RGBA);
    ok= ok && id_puntaje.Load("Texturas/puntaje.png", GL_RGBA);
    ok= ok && id_wasted.Load("Texturas/wasted.png", GL_RGBA);
    loadNumbersM(45,64);
    return ok;
}

void Menu::setCamara(float _x, float _y, float _z, float _motoX, float _motoY, float _motoZ){
    x = _x;
    y = _y;
    z = _z;
    motoX = _motoX;
    motoY = _motoY;
    motoZ = _motoZ;
}

void init2d(int w, int h){
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0, w, 0, h, -1, 1);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

void Menu::init3d(bool wireframe, bool textures, bool fullscreen){
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    if(fullscreen)
        gluPerspective(45, 1280/720.f, 0.1, 100);
    else
        gluPerspective(45, 1280/720.f, 0.1, 100);

    glEnable(GL_DEPTH_TEST);
    glMatrixMode(GL_MODELVIEW);
    gluLookAt(x,y,-z,motoX,motoY,motoZ,0,1,0);
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    if(textures)
        glEnable(GL_TEXTURE_2D);
    else
        glDisable(GL_TEXTURE_2D);
}

void DrawQuad(int x, int y, int w, int h){
    glDisable(GL_LIGHTING);
    glBegin(GL_QUADS);
        glTexCoord2d(0,0);
        glVertex2d(x,y);
        glTexCoord2d(1,0);
        glVertex2d(x+w,y);
        glTexCoord2d(1,1);
        glVertex2d(x+w,y+h);
        glTexCoord2d(0,1);
        glVertex2d(x,y+h);
    glEnd();
    glEnable(GL_LIGHTING);
}


void Menu::loadNumbersM(float ancho, float altura){
    glColor3f(1.0,1.0,1.0);

    float width = 1.0/11.0;
    float tu = 0;
    float tv = 0;

    for ( int i = 0; i < 10; i++) {
        int id=glGenLists(1);
        glNewList(id, GL_COMPILE);

        glBegin(GL_QUADS);
            glTexCoord2f(tu, 0);
            glVertex3f(0, 0, 0);
            glTexCoord2f(tu, 1);
            glVertex3f(0, altura, 0);
            glTexCoord2f(tu+width, 1);
            glVertex3f(ancho, altura, 0);
            glTexCoord2f(tu+width, 0);
            glVertex3f(ancho, 0, 0);
        glEnd();
        glEndList();

        tu = tu + width;

        switch ( i ) {
            case 0 : num.cero = id; break;
            case 1 : num.one = id; break;
            case 2 : num.two = id; break;
            case 3 : num.three = id; break;
            case 4 : num.four = id; break;
            case 5 : num.five = id; break;
            case 6 : num.six = id; break;
            case 7 : num.seven = id; break;
            case 8 : num.eight = id; break;
            case 9 : num.nine = id; break;
        }
    }

    TexLoader textura;
	textura.Load("Texturas/numeros.png", GL_RGBA);
	num.texID = textura.GetID();
}

void Menu::writeNumbersM(int points, float x, float y) {
    float ancho = 45;
    glPushMatrix();
        glTranslatef(x, y, 0);
        glEnable(GL_TEXTURE_2D);
        glBindTexture(GL_TEXTURE_2D,num.texID);
        int * digitos = new int[10];
        int i = 0;
        while (points > 0) {
            int resto = points % 10;
            points = points / 10;

            digitos[i] = resto;
            i++;
        }

        if (i == 0)
            digitos[i] = 0;
        else
            i--;
        while ( i >= 0) {
            switch ( digitos[i] ) {
                case 0 : glCallList(num.cero); break;
                case 1 : glCallList(num.one); break;
                case 2 : glCallList(num.two); break;
                case 3 : glCallList(num.three); break;
                case 4 : glCallList(num.four); break;
                case 5 : glCallList(num.five); break;
                case 6 : glCallList(num.six); break;
                case 7 : glCallList(num.seven); break;
                case 8 : glCallList(num.eight); break;
                case 9 : glCallList(num.nine); break;
            }
            glTranslatef(ancho, 0, 0);
            i--;
        }
        glDisable(GL_TEXTURE_2D);
    glPopMatrix();
}



void Menu::DrawInicio(int w, int h){
    glEnable(GL_TEXTURE_2D);
        glBindTexture(GL_TEXTURE_2D, id_comenzar.GetID());
        DrawQuad(w/2 - 128, margin_bot, 256, 64);

        glBindTexture(GL_TEXTURE_2D, id_flecha.GetID());
        DrawQuad(w-margin_side-64, h/2 - 32, 64, 64);
        DrawQuad(margin_side+64, h/2 - 32, -64, 64);
    glDisable(GL_TEXTURE_2D);
}


void Menu::runInicio(SDL_Window* window, vector<Moto*> motos, int& indice, bool& fin){
    bool iniciar= false;
    x=0;
    y=1;
    z=3;
    motoX= 0;
    motoY= 0;
    motoZ= 0;
    float rotacion= 0;
    float deltaTime = 0.0;
    int lastTime = 0;
    int thisTime = 0;


    int w,h;
    SDL_GetWindowSize(window,&w,&h);

    int idPlat;
    ObjLoader objL;
    objL.loadObj("Modelos/plataforma.obj", idPlat);

    indice= 0;
    SDL_Event evento;
    while(!iniciar){
        thisTime = SDL_GetTicks();
        deltaTime = (float)(thisTime - lastTime) / 1000;
        lastTime = thisTime;

        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        glLoadIdentity();

        init3d(false, true, true);
        glPushMatrix();
        glRotatef(rotacion, 0.0, 1.0, 0.0);
        motos[indice]->Draw();
        glCallList(idPlat);
        glPopMatrix();

        init2d(w, h);
        DrawInicio(w,h);

        while(SDL_PollEvent(&evento)){
            switch(evento.type){
                case SDL_QUIT:
                    fin = true;
                    iniciar= true;
                    break;
                case SDL_KEYDOWN:
                    switch(evento.key.keysym.sym){
                        case SDLK_RIGHT:
                            if(indice== motos.size()-1)
                                indice= 0;
                            else
                                indice++;
                        break;
                        case SDLK_LEFT:
                            if(indice== 0)
                                indice= motos.size()-1;
                            else
                                indice--;
                        break;
                        case SDLK_RETURN:
                            iniciar= true;
                            break;
                    }
                    break;
                case SDL_MOUSEBUTTONDOWN:
                    if(evento.button.y > h/2-32 && evento.button.y < h/2+32){
                        if(evento.button.x > margin_side && evento.button.x <  margin_side+64)
                            if(indice== 0)
                                indice= motos.size()-1;
                            else
                                indice--;
                        else if(evento.button.x > w-margin_side-64 && evento.button.x <  w-margin_side)
                            if(indice== motos.size()-1)
                                indice= 0;
                            else
                                indice++;
                    }
                    else if(evento.button.y > h-margin_bot-64 && evento.button.y < h-margin_bot && evento.button.x > w/2-128 && evento.button.x <  w/2+128)
                        iniciar= true;
                    break;
            }
        }
        if (1000.0/FPS > deltaTime)
            SDL_Delay(1000.0/FPS - deltaTime);

        rotacion= rotacion+0.5;
        SDL_GL_SwapWindow(window);
    }
    init3d(false, true, true);
}



void Menu::DrawPausa(int w, int h, bool& wireframe, bool& textures, bool& fin, bool& fullscreen){
    glEnable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_LIGHTING);

    glBindTexture(GL_TEXTURE_2D, 0);
        glBegin(GL_QUADS);
            glColor4f(0,0,0,0.5);
            glVertex2d(w/2 - 180,h/2 - 145);
            glVertex2d(w/2 + 180,h/2 - 145);
            glVertex2d(w/2 + 180,h/2 + 145);
            glVertex2d(w/2 - 180,h/2 + 145);
        glEnd();
        glColor4f(1, 1, 1, 1);

        glBindTexture(GL_TEXTURE_2D, id_pausa.GetID());
        DrawQuad(w/2 - 128, h/2+64, 256, 64);

        glBindTexture(GL_TEXTURE_2D, id_wireframe.GetID());
        DrawQuad(w/2 - 163, h/2, 256, 64);
        if(wireframe)
            glBindTexture(GL_TEXTURE_2D, id_tick.GetID());
        else
            glBindTexture(GL_TEXTURE_2D, id_notick.GetID());
        DrawQuad(w/2+93, h/2, 64, 64);

        glBindTexture(GL_TEXTURE_2D, id_texturas.GetID());
        DrawQuad(w/2 - 163, h/2-64, 256, 64);
        if(textures)
            glBindTexture(GL_TEXTURE_2D, id_tick.GetID());
        else
            glBindTexture(GL_TEXTURE_2D, id_notick.GetID());
        DrawQuad(w/2+93, h/2-64, 64, 64);

        glBindTexture(GL_TEXTURE_2D, id_fullscreen.GetID());
        DrawQuad(w/2 - 163, h/2-128, 256, 64);
        if(fullscreen)
            glBindTexture(GL_TEXTURE_2D, id_tick.GetID());
        else
            glBindTexture(GL_TEXTURE_2D, id_notick.GetID());
        DrawQuad(w/2+93, h/2-128, 64, 64);

    glBindTexture(GL_TEXTURE_2D, 0);
    glEnable(GL_DEPTH_TEST);
    glDisable(GL_BLEND);
    glDisable(GL_TEXTURE_2D);
}

void Menu::DrawWasted(int w, int h, int puntaje){
    glEnable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_LIGHTING);

    glBindTexture(GL_TEXTURE_2D, 0);
        glBegin(GL_QUADS);
            glColor4f(0,0,0,0.5);
            glVertex2d(w/2 - 272,h/2 - 128);
            glVertex2d(w/2 + 272,h/2 - 128);
            glVertex2d(w/2 + 272,h/2 + 128);
            glVertex2d(w/2 - 272,h/2 + 128);
        glEnd();
        glColor4f(1, 1, 1, 1);

        writeNumbersM(puntaje, w/2-8, h/2);
        glEnable(GL_TEXTURE_2D);

        glBindTexture(GL_TEXTURE_2D, id_wasted.GetID());
        DrawQuad(w/2 - 128, h/2+64, 256, 64);

        glBindTexture(GL_TEXTURE_2D, id_puntaje.GetID());
        DrawQuad(w/2 - 264, h/2, 256, 64);

        glBindTexture(GL_TEXTURE_2D, id_reintentar.GetID());
        DrawQuad(w/2 - 264, h/2-120, 256, 64);

        glBindTexture(GL_TEXTURE_2D, id_salir.GetID());
        DrawQuad(w/2 + 8, h/2-120, 256, 64);
        glDisable(GL_TEXTURE_2D);

    glBindTexture(GL_TEXTURE_2D, 0);
    glEnable(GL_DEPTH_TEST);
    glDisable(GL_BLEND);
    glDisable(GL_TEXTURE_2D);
}


void Menu::runPausa(SDL_Window* window, Mapa* mapa, Moto* moto, bool& wireframe, bool& textures, bool& fin, bool& fullscreen){
    bool pausa= true;
    int wW,hW;
    SDL_GetWindowSize(window,&wW,&hW);

    do{
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        glLoadIdentity();
        init3d(wireframe, textures, fullscreen);
        mapa->Draw(x,y,-z,0);
        moto->Draw();
        init2d(wW,hW);
        glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
        DrawPausa(wW, hW, wireframe, textures, fin, fullscreen);
        if(wireframe)
            glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );

        SDL_Event evento;
        while(SDL_PollEvent(&evento)){
            switch(evento.type){
                case SDL_QUIT:
                    fin = true;
                    pausa = false;
                    break;
                case SDL_KEYDOWN:
                    switch(evento.key.keysym.sym){
                        case SDLK_ESCAPE:
                            pausa = false;
                            break;
                        case SDLK_p:
                            pausa= false;
                            break;
                    }
                    break;
                case SDL_MOUSEBUTTONDOWN:
                    if(evento.button.x > wW/2+93 && evento.button.x < wW/2+157){
                        if(evento.button.y > hW/2-64 && evento.button.y <  hW/2)
                            wireframe= !wireframe;
                        else if(evento.button.y > hW/2 && evento.button.y < hW/2+64)
                            textures= !textures;
                        else if(evento.button.y > hW/2+64 && evento.button.y <  hW/2+128)
                            fullscreen= !fullscreen;
                    }
                    break;
            }
        }
        SDL_GL_SwapWindow(window);
    }while(pausa);
    init3d(wireframe, textures, fullscreen);
}


void Menu::runWasted(SDL_Window* window, bool& wasted, Mapa* mapa, Moto* moto, bool& wireframe, bool& textures, bool& fin, bool& fullscreen, int puntos){
    int wW,hW;
    SDL_GetWindowSize(window,&wW,&hW);

    do{
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        glLoadIdentity();
        init3d(wireframe, textures, fullscreen);
        mapa->Draw(x,y,-z,0);
        moto->Draw();
        init2d(wW,hW);
        glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
        DrawWasted(wW, hW, puntos);
        if(wireframe)
            glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );

        SDL_Event evento;
        while(SDL_PollEvent(&evento)){
            switch(evento.type){
                case SDL_QUIT:
                    fin = true;
                    wasted = false;
                    break;
                case SDL_MOUSEBUTTONDOWN:
                    if(evento.button.y > hW/2+56 && evento.button.y < hW/2+120){
                        if(evento.button.x > wW/2-264 && evento.button.x <  wW/2-8)
                            wasted=  false;
                        else if(evento.button.x > wW/2+8 && evento.button.x <  wW/2+264){
                            wasted=  false;
                            fin= true;
                        }
                    }
                    break;
            }
        }
        SDL_GL_SwapWindow(window);
    }while(wasted);
    init3d(wireframe, textures, fullscreen);
}



/*
    TTF_Font *times;
    times = TTF_OpenFont("Times.ttf", 8);
    if(!times)
        cerr << "shittt" << endl;
    SDL_Color cl = {0, 255, 0};
    SDL_Surface * mensaje = TTF_RenderUTF8_Blended(times,"HOLASSDDD",cl);

    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


    int colors = mensaje->format->BytesPerPixel;
    GLuint texture_format;
    if (colors == 4) {   // alpha
        if (mensaje->format->Rmask == 0x000000ff)
            texture_format = GL_RGBA;
        else
            texture_format = GL_BGRA;
    } else {             // no alpha
        if (mensaje->format->Rmask == 0x000000ff)
            texture_format = GL_RGB;
        else
            texture_format = GL_BGR;
    }
    GLuint textura;
    glGenTextures(1, &textura);
    glBindTexture(GL_TEXTURE_2D, textura);
    glTexImage2D(GL_TEXTURE_2D, 0, colors, mensaje->w, mensaje->h, 0, texture_format , GL_UNSIGNED_BYTE, mensaje->pixels);

*/
