#include "ObjLoader.h"
#include <fstream>
#include <iostream>
#include <sstream>
#include <glm/glm.hpp>
#include <algorithm>
#include "SDL2/SDL_opengl.h"
#include "FreeImage/FreeImage.h"
#include "TexLoader.h"
#define GL_GLEXT_PROTOTYPES
#include <cstring>


ObjLoader::ObjLoader()
{
    //ctor
}

ObjLoader::~ObjLoader()
{
    //dtor
}

struct material{
	string name;
	glm::vec3 Ka;
	glm::vec3 Kd;
	glm::vec3 Ks;
	int illum;
	float Nsa;
	int map_Ka;
	int map_Kd;

	material(string _name, glm::vec3 _Ka,	glm::vec3 _Kd,glm::vec3 _Ks,
	int a_illum,	float a_Ns,	string _map_Ka,	string _map_Kd){
		name = _name;
		Ka = _Ka;
		Kd = _Kd;
		Ks = _Ks;
		illum = a_illum;
		this ->Nsa = a_Ns;
		char* Char_map_Ka = new char[_map_Ka.length()+1];
                strcpy(Char_map_Ka, (_map_Ka).c_str());
        char* Char_map_Kd = new char[_map_Kd.length()+1];
                strcpy(Char_map_Kd, (_map_Kd).c_str());
        if(_map_Ka.compare("")==0)
            map_Ka=-1;
        else{
            TexLoader tex;
            tex.Load(Char_map_Ka, GL_RGB);
            map_Ka=tex.GetID();
        }

        if(_map_Kd.compare("")==0)
            map_Kd=-1;
        else{
            TexLoader tex1;
            tex1.Load(Char_map_Kd, GL_RGB);
            map_Kd=tex1.GetID();
        }
	}
};



void LoadMaterials(const char* filename, vector<material*>& materiales){
ifstream in(filename, ios::in);
    if (!in)
    {
        cerr << "Imposible abrir archivo: " << filename << endl;
        return;// false;
    }
    else{
		bool first = true;

		string name;
		glm::vec3 Ka;
		glm::vec3 Kd;
		glm::vec3 Ks;
		int illum=0;
		float Nss=0.0;
		string map_Ka;
		string map_Kd;

        //Leer el archivo
        string line;
        //Leer todas las lineas
        while (getline(in, line)){
            //Nuevo material
            if (line.substr(0,6) == "newmtl"){
				if (!first)
                    materiales.push_back(new material( name, Ka, Kd, Ks, illum,	Nss,	map_Ka,	map_Kd));
				else
					first =false;
				name = line.substr(7);
				//Le doy valores estandar a todos los parametros
				Ka.x = 1.0; Ka.y=1.0;Ka.z=1.0;
				Kd.x = 1.0; Kd.y=1.0;Kd.z=1.0;
				Ks.x = 1.0; Ks.y=1.0;Ks.z=1.0;
				illum = 2;
				Nss = 64;
				map_Ka = -1;
				map_Kd = -1;
            }
            //ambient color(rgb)
            else if (line.substr(0,2) == "Ka"){
                istringstream s(line.substr(2));
                s >> Ka.x;
                s >> Ka.y;
                s >> Ka.z;
            }
            //diffuse color(rgb)
            else if (line.substr(0,2) == "Kd"){
                istringstream s(line.substr(2));
                s >> Kd.x;
                s >> Kd.y;
                s >> Kd.z;
            }
			//specular color(rgb)
            else if (line.substr(0,2) == "Ks"){
                istringstream s(line.substr(2));
                s >> Ks.x;
                s >> Ks.y;
                s >> Ks.z;
            }
            //Iluminacion del modelo 1= flat sin specular 2=con especular
            else if (line.substr(0,5) == "illum"){
                illum =  atoi(line.substr(6).c_str());
            }
            //Brillo del modelo
			else if (line.substr(0,2) == "Ns"){
                Nss =  atof(line.substr(2).c_str());
            }
            else if (line.substr(0,6) == "map_Ka"){
				map_Ka = line.substr(6);
            }
			else if (line.substr(0,6) == "map_Kd"){
				map_Kd = line.substr(7);
            }
            //Ignorar
            else{
            }

        }
        materiales.push_back(new material( name, Ka, Kd, Ks, illum,	Nss,	map_Ka,	map_Kd));
        in.close();
        in.clear();
    }
}

bool ObjLoader::loadObj(const char* filename, int& idObj){

    //Abrir el archivo////////////////////////////////////////////////////////////
    ifstream in(filename, ios::in);
    if (!in)
    {
        cerr << "Imposible abrir archivo: " << filename << endl;
        return false;
    }
    else{
        vector<glm::vec3> vertices;
        vector<glm::vec3> normales;
        vector<unsigned short> caras;
        vector<glm::vec2> uvs;
        vector<unsigned short> materialesI;
        vector<string> materialesN;

        vector<material*> materiales;
        bool hasMaterial = false;

        unsigned short faces = 0;

        //Leer el archivo obj/////////////////////////////////////////////////////
        string line;

        //Leer todas las lineas
        while (getline(in, line)){
            //Agregar verices
            if (line.substr(0,2) == "v "){
                istringstream s(line.substr(2));
                glm::vec3 v;
                s >> v.x;
                s >> v.y;
                s >> v.z;
                vertices.push_back(v);
            }
            //Agregar normales
            else if (line.substr(0,2) == "vn"){
                istringstream s(line.substr(2));
                glm::vec3 v;
                s >> v.x;
                s >> v.y;
                s >> v.z;
               normales.push_back(v);
            }
            //Agregar coordenadas de textura
            else if (line.substr(0,2) == "vt"){
                istringstream s(line.substr(2));
                glm::vec2 v;
                s >> v.x;
                s >> v.y;
                uvs.push_back(v);
            }
            //Agrega la lista de materiales
             else if (line.substr(0,6) == "mtllib"){
                    hasMaterial=true;
                    char* mat_Archivo = new char[line.substr(7).length()+1];
                strcpy(mat_Archivo, (line.substr(7)).c_str());
                    LoadMaterials(mat_Archivo, materiales);
            }
            //Agregar el nombre del material e indice de la cara en la que comienza
            else if (line.substr(0,6) == "usemtl"){
                unsigned short i = faces;
                materialesN.push_back(line.substr(7));
                materialesI.push_back(i);
            }
            //Agregar caras
            else if (line.substr(0,2) == "f "){
                faces++;
                //Si no hay materiales lee indices de vertices y notmales
                if(materialesI.size() == 0){
                    unsigned short cara[6];
                    string str = line.substr(2);
                    replace( str.begin(), str.end(), '/', ' ');
                    istringstream s(str);
                    for(int i=0; i<6; i++){
                        s >> cara[i];
                    }
                    //Agregar: indice de normal - indice de vertice
                    for(int j=0; j<3; j++){
                        caras.push_back(cara[2*j+1]-1);
                        caras.push_back(cara[2*j]-1);
                    }
                }
                //Si hay materiales leer indices de vertices, normales y texturas
                else{
                    unsigned short cara[9];
                    string str= line.substr(2);
                    replace( str.begin(), str.end(), '/', ' ');
                    istringstream s(str);
                    for(int i=0; i<9; i++){
                        s >> cara[i];
                    }
                    //Agregar: indice de normal - indice de vertice - indice de textura
                    for(int j=0; j<3; j++){
                        caras.push_back(cara[3*j+2]-1);
                        caras.push_back(cara[3*j]-1);
                        caras.push_back(cara[3*j+1]-1);
                    }
                }
            }
            //Ignorar
            else{
            }
        }
        in.close();
        in.clear();


        //Dibujo el objeto en la lista////////////////////////////////////////////
        idObj = glGenLists(1);

        //Si el modelo NO tiene texturas
        if(materialesI.size()==0){
            GLfloat ambientColor[] = {0.5, 0.5, 0.5, 1.0};
            GLfloat difusseColor[] = {0.5, 0.5, 0.5, 1.0};
            GLfloat specularColor[] = { 0.0,  0.0,  0.0, 1.0};

            glNewList(idObj, GL_COMPILE);

            glMaterialfv(GL_FRONT, GL_DIFFUSE,difusseColor);
                glMaterialfv(GL_FRONT,GL_AMBIENT,ambientColor);
                glMaterialfv(GL_FRONT,GL_SPECULAR,specularColor);
                glMaterialf(GL_FRONT,GL_SHININESS,64);


            glBegin(GL_TRIANGLES);
                for(unsigned int i=0; i<caras.size()/6; i++){
                    glNormal3d(normales[caras[i*6]].x, normales[caras[i*6]].y, normales[caras[i*6]].z);
                    glVertex3d(vertices[caras[(i*6)+1]].x, vertices[caras[(i*6)+1]].y, vertices[caras[(i*6)+1]].z);

                    glNormal3d(normales[caras[(i*6)+2]].x, normales[caras[(i*6)+2]].y, normales[caras[(i*6)+2]].z);
                    glVertex3d(vertices[caras[(i*6)+3]].x, vertices[caras[(i*6)+3]].y, vertices[caras[(i*6)+3]].z);

                    glNormal3d(normales[caras[(i*6)+4]].x, normales[caras[(i*6)+4]].y, normales[caras[(i*6)+4]].z);
                    glVertex3d(vertices[caras[(i*6)+5]].x, vertices[caras[(i*6)+5]].y, vertices[caras[(i*6)+5]].z);
                }
            glEnd();
            glEndList();
        }
        //Si el modelo tiene texturas
        else{
            int i = 0;
            int lim;
            glNewList(idObj, GL_COMPILE);
                for(int t = 0; t < materialesI.size(); t++){
                    if(t<materialesI.size()-1)
                        lim = materialesI[t+1];
                    else
                        lim= caras.size()/9;

                    material* _material;

                    //busco el material
                    for(int m = 0; m < materiales.size(); m++){
                        if((materiales[m]->name).compare(materialesN[t])==0){
                            _material = materiales[m];
                            break;
                        }
                    }
                    GLfloat ambientColor[] = {_material->Ka.x, _material->Ka.y, _material->Ka.z, 1.0};
                    GLfloat difusseColor[] = {_material->Kd.x, _material->Kd.y, _material->Kd.z, 1.0};
                    GLfloat specularColor[] = {_material->Ks.x, _material->Ks.y, _material->Ks.z, 1.0};




                    glMaterialfv(GL_FRONT, GL_DIFFUSE,difusseColor);
                    glMaterialfv(GL_FRONT,GL_AMBIENT,ambientColor);
                    glMaterialfv(GL_FRONT,GL_SPECULAR,specularColor);
                    glMaterialf(GL_FRONT,GL_SHININESS,_material->Nsa);
                    if(_material->map_Kd!=-1){
                        glBindTexture(GL_TEXTURE_2D, _material->map_Kd);
                        glBegin(GL_TRIANGLES);
                    }

                    for( ; i < lim; i++){
                        glTexCoord2d(uvs[caras[(i*9)+2]].x,uvs[caras[(i*9)+2]].y);
                        glNormal3d(normales[caras[i*9]].x, normales[caras[i*9]].y, normales[caras[i*9]].z);
                        glVertex3d(vertices[caras[(i*9)+1]].x, vertices[caras[(i*9)+1]].y, vertices[caras[(i*9)+1]].z);

                        glTexCoord2d(uvs[caras[(i*9)+5]].x,uvs[caras[(i*9)+5]].y);
                        glNormal3d(normales[caras[(i*9)+3]].x, normales[caras[(i*9)+3]].y, normales[caras[(i*9)+3]].z);
                        glVertex3d(vertices[caras[(i*9)+4]].x, vertices[caras[(i*9)+4]].y, vertices[caras[(i*9)+4]].z);

                        glTexCoord2d(uvs[caras[(i*9)+8]].x,uvs[caras[(i*9)+8]].y);
                        glNormal3d(normales[caras[(i*9)+6]].x, normales[caras[(i*9)+6]].y, normales[caras[(i*9)+6]].z);
                        glVertex3d(vertices[caras[(i*9)+7]].x, vertices[caras[(i*9)+7]].y, vertices[caras[(i*9)+7]].z);
                    }
                    glEnd();
                    glBindTexture(GL_TEXTURE_2D,0);
                    glColor4f(1,1,1,1);
                }
            glEndList();
           // glColor3f(1.0,1.0,1.0);
        }
        return true;
    }
}







bool ObjLoader::loadTerreno(const char* filename, const char* heightMap, int& idMap, vector<float> &alturas){

    this->loadObj(filename, idMap);
    //Abrir el archivo////////////////////////////////////////////////////////////

    ifstream in(heightMap, ios::in);
    if (!in)
    {
        cerr << "Imposible abrir archivo: " << filename << endl;
        return false;
    }
    else{
        vector<glm::vec2> temp;
        //Leer el archivo obj/////////////////////////////////////////////////////
        string line;
        //Leer todas las lineas
        while (getline(in, line)){
            //Agregar verices
            if (line.substr(0,2) == "v "){
                istringstream s(line.substr(2));
                glm::vec3 v;
                s >> v.x;
                s >> v.y;
                s >> v.z;

                glm::vec2 w;
                w.x= v.x;
                w.y= v.y;
                temp.push_back(w);

            }
        }
        in.close();
        in.clear();

        //alturas= new vector<float>[temp.size()];
        for(int i=0; i<temp.size(); i++){
            alturas.push_back(1.0);
        }

        //Ordenar los vertices segun la coordenada X
        for(int i=0; i<temp.size(); i++){
            float h= temp[i].y;
            //El mapa tiene un vertice cada 0.5 de distancia en X
            int indice=int(temp[i].x * 2);
            alturas[indice]= h;
            //alturas.push_back(i*1.0);
        }
        return true;
    }
}
